const relocateCursor = (editor, isPaste) => {
  const returnCondition = 'figcaption';

  const sel = editor.selection.getSel();
  const node = editor.selection.getNode();

  // 붙여넣기의 경우는 아래와 같은 경우가 없지만, 다른 플러그인에서 relocateCursor만을 사용할 경우 발생 가능
  if(!sel.anchorNode && !sel.focusNode ) {
    return;
  }

  const fOffset = sel.focusOffset;
  sel.collapseToEnd();

  const targetNode = tinymce.dom.DomQuery(node).closest('body > *, body > div > *')[0] || sel.focusNode.childNodes[fOffset-1];
  if(!targetNode) {
    return;
  }

  if(isPaste) {
    if(editor.dom.is(node, returnCondition)) {
      return node;
    }
    if(!editor.dom.is(targetNode, 'p, h1, h2, h3, table, div, blockquote')) {
      insertParagraph(editor, targetNode);
    }
  }
  else {
    if(!editor.dom.is(targetNode, 'p, h1, h2, h3, div')) {
      insertParagraph(editor, targetNode);
    }
  }
  return targetNode;
}

const insertParagraph = (editor, node) => {
  editor.undoManager.ignore(function() {
    const p = document.createElement('p');
    editor.dom.insertAfter(p, node);
    const range = editor.selection.getRng()
    range.setStart(p, 0)
    range.setEnd(p, 0)
    editor.selection.setCursorLocation(p)
    editor.selection.setRng(range);
    // editor.dom.remove(p);
  })
}

export default relocateCursor;
