const plugin = (editor) => {
  const settings = editor.settings.autosave;

  editor.on('init', () => {
    if(localStorage.getItem('autosaved')) {
      console.log("there's automatically saved post.")
    }

    if(settings) {
      if(settings.is_interval_save === 0) {
        noTimeInterval();
      }
      else if(settings.is_interval_save === 1){
        timeInterval();
      }
      else {
        debouncing();
      }
    }
  });

  const noTimeInterval = () => {
    let shouldSave = true;
    editor.on('change', (e) => {
      if(e.originalEvent) {
        shouldSave = true;
      }
      if(!shouldSave) {
        shouldSave = true;
        return;
      }
      saveContent();
      if(!e.originalEvent) {
        shouldSave = false;
      }
    });
  }

  const timeInterval = () => {
    let lastSavedTime = 1;
    editor.on('change', (e) => {
      let now = Date.now();
      if(now - lastSavedTime > editor.settings.autosave.save_interval) {
        lastSavedTime = now;
        saveContent();
      }
    })
  }

  const debouncing = () => {
    let keystrokeTimeout = undefined;
    editor.on('KeyDown', (e) => {
      if(keystrokeTimeout) {
        clearTimeout(keystrokeTimeout);
      }
      keystrokeTimeout = setTimeout(saveContent, 5000);
    })

    editor.on('change', (e) => {
      if(e.originalEvent) {
        saveContent();
      }
    })
  }

  const saveContent = () => {
    // localStorage.autosaved = editor.getContent();
    console.log("SAVED");
  }
}

export default plugin;
