const plugin = editor => {
  const updateSelection = function(editor) {
    return function(e) {
      const blockquoteStyleType = editor.selection.getNode().getAttribute('data-ke-style')
      e.control.items().each(function(ctrl) {
        ctrl.active(ctrl.settings.value === blockquoteStyleType)
      })
    }
  }

  const createFormatList = function(editor) {
    let count = 1
    const newFormats = []

    const createMenu = function(formats: Array<any>) {
      const menu = []

      if (!formats) {
        return
      }

      formats.forEach(function(format) {
        const menuItem: any = {
          text: format.title,
          icon: format.icon,
        }

        if (format.items) {
          menuItem.menu = createMenu(format.items)
        } else {
          const formatName = format.format || 'style' + count++

          if (!format.format) {
            format.name = formatName
            newFormats.push(format)
          }

          menuItem.format = formatName
          menuItem.cmd = format.cmd
        }

        menu.push(menuItem)
      })

      return menu
    }

    editor.on('init', function() {
      newFormats.forEach(function(format) {
        editor.formatter.register(format.name, format)
      })
    })

    return createMenu(editor.settings.blockquoteList)
  }

  const checkNode = function(parentsList) {
    return parentsList && parentsList[parentsList.length - 1]
  }

  const registerButtons = function(editor, list) {
    const items = []

    list.forEach(function(block) {
      items.push({
        text: block.text,
        value: block.format,
        icon: block.icon,
        textStyle() {
          return editor.formatter.getCssText(block.format)
        },
      })
    })
    editor.addButton('blockquoteList', {
      type: 'simplemenubutton',
      icon: 'blockquoteList',
      tooltip: 'Blockquote',
      fixedWidth: true,
      menu: {
        items,
        onPostRender(e) {
          e.control.fixed(true)
        },
      },
      onPostRender(e) {
        const ctrl = e.control
        editor.on('NodeChange', function(e) {
          if (checkNode(e.parents)) {
            ctrl.active(e.parents[e.parents.length - 1].nodeName === 'BLOCKQUOTE')
          }
        })
      },
      onshow: updateSelection(editor),
      onselect(e) {
        if (e.control) {
          const selectNode: Node = editor.selection.getNode()
          if (['IMG', 'FIGURE', 'FIGCAPTION'].indexOf(selectNode.nodeName) == -1) {
            editor.fire('BeforeSetContent')
            editor.formatter.apply(e.control.value())
            editor.undoManager.add()
          }
        }
      },
      onhide(e) {
        if (editor.dom.is(editor.selection.getNode(), 'blockquote')) {
          e.control.parents('*')[0].classes.toggle('active', true)
        }
      },
    })
  }

  const formatList = createFormatList(editor)
  registerButtons(editor, formatList)
}

export default plugin
