import CodeEditor from './CodeEditor'
import autobind from 'autobind-decorator'
import { ENGINE_METHOD_DIGESTS } from 'constants'

class CodeEditorDialog {
  constructor(editor) {
    this.editor = editor
    this.active = false
  }

  open() {
    if (this.active) {
      return
    }

    if (!this.$container) {
      this.createContainer()
    }

    this.$container.show()
    this.$container.offset()
    this.$container.addClass('mce-in')
    this.codeEditor.refresh()
    this.active = true
  }

  close() {
    if (!this.active) {
      return
    }

    this.$container.removeClass('mce-in')
    this.$container.offset()
    this.$container.hide()

    this.active = false
  }

  @autobind
  handleClose(e) {
    this.close()
  }

  createContainer() {
    const preferrenceText = localStorage.getItem('keditor_codeblock_preferrence_text')
    const preferrenceValue = localStorage.getItem('keditor_codeblock_preferrence_value')
    const { $, dom } = this.editor
    const listbox = tinymce.ui.Factory.create({
      type: 'listbox',
      menu: {
        items: this.editor.settings.codeblock.languages,
        onPostRender(e) {
          e.control.fixed(true)
        },
      },
      text: preferrenceText ? preferrenceText : this.editor.settings.codeblock.languages[0].text,
      value: preferrenceValue ? preferrenceValue : this.editor.settings.codeblock.languages[0].value,

      onshow(e) {
        // 다이얼로그가 tinymce window가 아니라 z-index를 동적으로 계산하는 과정에서 65535로 고정이 되어버리는 문제.
        Object.values(listbox._eventsRoot.controlIdLookup)
          .filter(v => v.settings.type === 'menu')[0]
          .$el.css('z-index', '65537')
      },
      onselect(e) {
        listbox.value(e.control.settings.value)
        listbox.text(e.control.settings.text)
      },
    })

    let container = dom.add(
      document.body,
      'div',
      { class: 'mce-codeblock-dialog-container mce-window' },
      `
    <div class="mce-codeblock-shadow"></div>
    <div class="mce-codeblock-dialog">
      <div class="mce-codeblock-header mce-window-head">
        <div class="mce-title">
          코드블럭 삽입
          <div class="mce-codeblock-select-wrapper">
            
          </div>
        </div>
        
        <button class="mce-close"><i class="mce-ico mce-i-remove"></i></button>
      </div>
      <div class="mce-codeblock-content">
        <textarea class="textarea"></textarea>
      </div>
      <div class="mce-foot mce-container">
        <div class="mce-container-body">
          <div class="mce-btn mce-codeblock-btn-cancel">
            <button><span class="mce-txt">취소</span></button>
          </div>
          <div class="mce-btn mce-primary mce-codeblock-btn-submit">
            <button><span class="mce-txt">확인</span></button>
          </div>
        </div>
      </div>
    </div>
    `
    )

    const $container = $(container)
    listbox.renderTo($container.find('.mce-codeblock-select-wrapper')[0])

    const $shadow = $container.find('.mce-codeblock-shadow')
    const $btnClose = $container.find('.mce-close')
    const $btnCancel = $container.find('.mce-codeblock-btn-cancel button')
    const $btnSubmit = $container.find('.mce-codeblock-btn-submit button')
    const $textarea = $container.find('.textarea')

    this.codeEditor = new CodeEditor(this.editor, $textarea[0])
    // this.codeEditor.onChange(value => {
    //   if (value && value.length > 0) {
    //     $btnSubmit.attr('disabled', null)
    //   } else {
    //     $btnSubmit.attr('disabled', true)
    //   }
    // })

    $btnSubmit.on('click', () => {
      // if (!this.active) {
      //   return
      // }

      localStorage.setItem('keditor_codeblock_preferrence_text', listbox.state.data.text)
      localStorage.setItem('keditor_codeblock_preferrence_value', listbox.state.data.value)
      const result = this.codeEditor.insertCodeBlock(listbox.state.data.value)
      if (!result) {
        alert('코드를 입력해 주세요.')
      } else {
        this.close()
      }
    })

    $btnClose.on('click', this.handleClose)
    $btnCancel.on('click', this.handleClose)
    $shadow.on('click', this.handleClose)

    this.$container = $container
  }
}

export default CodeEditorDialog
