import PalettePanel from './components/action/PalettePanelAction'

const plugin = editor => {
  const settings = editor.settings.colorPalette
  const hilite = settings && settings.hilite
  const align = settings && settings.align
  const forecolorPalettePanel = new PalettePanel(editor)
  const hilitecolorPalettePanel = new PalettePanel(editor)

  const setButtonColor = function() {
    return function() {
      const control = this
      editor.on('nodeChange', function() {
        const selectedNode = editor.selection.getSel().anchorNode
        const parentSpans = editor.dom.getParents(selectedNode, 'span')
        const format = control.settings.format
        let parentSpan: { style: { color: String; backgroundColor: String } }
        for (let span of parentSpans) {
          if (format === 'forecolor' && span.id !== '_mce_caret' && span.style && span.style.color) {
            parentSpan = span
            break
          } else if (format === 'hilitecolor' && span.id !== '_mce_caret' && span.style && span.style.backgroundColor) {
            parentSpan = span
            break
          }
        }
        let color: String = format === 'forecolor' ? settings.default_color : '#ffffff'
        if (parentSpan) {
          color = format === 'forecolor' ? parentSpan.style.color : parentSpan.style.backgroundColor
          color = forecolorPalettePanel.convertToHexCode(color)
        }
        control.color(color)
        const $iconElem = editor.$('i', control.$el[0])
        if (format === 'hilitecolor' && settings && settings.black_text_color_background) {
          if (settings.black_text_color_background.includes(color)) {
            $iconElem.removeClass('white-text')
          } else {
            $iconElem.addClass('white-text')
          }
        }
      })
    }
  }

  editor.addButton('colorPalette', {
    icon: 'forecolor',
    classes: 'forecolor',
    format: 'forecolor',
    type: 'colorbutton',
    tooltip: '글자색',
    popoverAlign: align === 'center' ? ['bc-tc'] : ['bc-tl'],
    panel: {
      role: 'dialog',
      ariaRemember: true,
      html: generatePalette(settings, 'forecolor'),
      onPostRender(e) {
        this.fixed(true)
        forecolorPalettePanel.init(this)
      },
    },
    onclick() {
      if (this.classes.contains('opened')) {
        this.hidePanel()
      } else {
        this.showPanel()
      }
      hilitecolorPalettePanel.panel && hilitecolorPalettePanel.hide()
    },
    onPostRender: setButtonColor(),
    onshow() {
      forecolorPalettePanel.show()
    },
    onhide() {
      forecolorPalettePanel.hide()
    },
  })

  if (hilite) {
    editor.addButton('backColorPalette', {
      icon: 'hilitecolor',
      classes: 'hilitecolor',
      format: 'hilitecolor',
      type: 'colorbutton',
      tooltip: '배경색',
      popoverAlign: align === 'center' ? ['bc-tc'] : ['bc-tl'],
      panel: {
        role: 'dialog',
        ariaRemember: true,
        html: generatePalette(settings, 'hilitecolor'),
        onPostRender(e) {
          this.fixed(true)
          hilitecolorPalettePanel.init(this)
        },
      },
      onclick() {
        if (this.classes.contains('opened')) {
          this.hidePanel()
        } else {
          this.showPanel()
        }
        forecolorPalettePanel.panel && forecolorPalettePanel.hide()
      },
      onPostRender: setButtonColor(),
      onshow() {
        hilitecolorPalettePanel.show()
      },
      onhide() {
        hilitecolorPalettePanel.hide()
      },
    })
  }
}

const generatePalette = (settings, format) => {
  const {
    module: modules,
    color_map_preset: colorMapPreset,
    color_map_palette: colorMapPalette,
    preset_rows: presetRows,
    preset_cols: presetCols,
    palette_rows: paletteRows,
    palette_cols: paletteCols,
  } = settings
  const additionalClass =
    format === 'hilitecolor' ? 'hilite-colorPalette-preset-td' : 'forecolor-colorPalette-preset-td'
  let html = ''

  if (modules.includes('preset')) {
    html += "<div class='colorPalette-div colorPalette-div-preset'><table class='colorPalette-preset-table'>"
    for (let i = 0; i < presetRows; i++) {
      html += "<tr class='colorPalette-preset-tr'>"
      for (let j = 0; j < presetCols; j++) {
        const curColor = colorMapPreset[i * presetCols + j]
        const removeFormatClass = curColor === '' ? ' remove-format' : ''
        const whiteColorClass = curColor === '#fff' || curColor === '#ffffff' ? ' white-color-format' : ''
        html += `<td class='colorPalette-preset-td ${additionalClass}${removeFormatClass}${whiteColorClass}' id=colorPalette-${format}-preset-id-${i *
          presetCols +
          j} style='background:${curColor}' data-mce-color=${curColor}><span class='colorPalette-preset-check'></span></td>`
      }
      html += '</tr>'
    }
    html += '</table></div>'
  }

  if (modules.includes('palette')) {
    html += "<div class='colorPalette-div colorPalette-div-palette'><table class='colorPalette-palette-table'>"
    for (let i = 0; i < paletteRows; i++) {
      html += "<tr class='colorPalette-palette-tr'>"
      for (let j = 0; j < paletteCols; j++) {
        const curColor = colorMapPalette[i * paletteCols + j]
        const removeFormatClass = curColor === '' ? ' remove-format' : ''
        const whiteColorClass = curColor === '#fff' || curColor === '#ffffff' ? ' white-color-format' : ''
        html += `<td class='colorPalette-palette-td ${additionalClass}${removeFormatClass}${whiteColorClass}' id=colorPalette-${format}-palette-id-${i *
          14 +
          j} style='background:${curColor};' data-mce-color=${curColor}></td>`
      }
      html += '</tr>'
    }
    html += '</table></div>'
  }

  if (modules.includes('code')) {
    html += "<div class='colorPalette-div colorPalette-div-code'>"
    html +=
      "<div class='colorPalette-code-display'></div><input maxlength='7' class='colorPalette-code-input'></input><button class='colorPalette-code-submit'>입력</button></div>"
  }

  return html
}

export default plugin
