const plugin = (editor) => {

  const key = editor.settings.emoticon.api_key;

  editor.on('init', () => {
    const win = editor.getWin();
    const $ = editor.dom.$;
    const $head = $('head');
    const emoticonCss = editor.dom.create('link');
    emoticonCss.rel = "stylesheet";
    emoticonCss.type = "text/css";
    emoticonCss.href = "//mk.kakaocdn.net/dn/emoticon/static/stylesheets/sdk/1.1.3/emoticon-1.1.3.css";

    const emoticonScript = editor.dom.create('script', {
      type: 'text/javascript',
      src: '//mk.kakaocdn.net/dn/emoticon/static/javascripts/sdk/1.3.7/sandbox/kakao-1.3.7-k.js'
    });
    emoticonScript.onload = () => {
      const kakaoLoader = editor.dom.create('script', {
        type: 'text/javascript',
      });
      kakaoLoader.innerHTML = `Kakao.init('${key}');`
      $head.append(kakaoLoader);
    }
    $head.append(emoticonCss);
    $head.append(emoticonScript);
  })

  editor.addButton('logout', {
    icon: 'forecolor',
    onclick(e) {
      const pr = new Promise(res => {
        Kakao.Auth.logout(() => {
          res();
        });  
      });
      pr.then(() => {
        window.location.href = `//accounts.kakao.com/logout?continue=${window.location.href}`;
      });
    }
  })

  editor.addButton('emoticon', {
    icon: 'forecolor',
    type: 'panelbutton',
    tooltip: '이모티콘',
    popoverAlign: ['bc-tl'],
    panel: {
      role: 'dialog',
      ariaRemember: true,
      html: generateKeyboard(),
      onPostRender() {
        this.fixed(true)
      }
    },
    onclick(e) {
      if(!Kakao.Emoticon.emoticon.options.callback.hidePanel) {
        Kakao.Emoticon.emoticon.options.callback.hidePanel = this.hidePanel.bind(this);
      }
      toggleMenu(e,'keyboardLayer');
    },
  });

  Kakao.init(key);

  Kakao.Emoticon.init({
    tabmenu: 'TOP',
    needConnectKakao: true,
    callback: {
      'showKeyboard': function () {},
      'hideKeyboard': function (e) {
        Kakao.Emoticon.showKeyboard(document.getElementById('keyboardLayer'));
      },
      'selectEmoticon': function (obj) {
        getCallback(obj);
        this.hidePanel();
      },
      'connectKakao': function () {
        Kakao.Auth.login({
          success: function(res) {
            console.log('LOG IN: ', res);
          }
        });
      }
    }
  });
  // Kakao.Emoticon.emoticon._onDocumentClicked = null;

  function getCallback (obj) {
    const div = document.createElement('div');
    div.classList.add('kakao_emoticon');
    div.classList.add('e_cover');
    const i = document.createElement('img');
    i.setAttribute('src', obj.image_url);
    i.setAttribute('data-id', obj.item_id);
    i.setAttribute('data-idx', obj.resource_id);
    i.setAttribute('data-type', obj.item_sub_type);
    i.style.width = "100px";
    i.style.height = "100px";
    i.classList.add("thumb_img");
    div.appendChild(i);
    editor.insertContent(div.outerHTML);
    editor.getWin().Kakao.Emoticon.update('kakao_emoticon');
  }

  function toggleMenu (e, keyboardContainer) {
    Kakao.Emoticon.showKeyboard(document.getElementById(keyboardContainer));
  }
}
const generateKeyboard = () => {
  return `<div class="keyboard_wrapper" style="width: 386px;">
            <div id="keyboardLayer" class="keyboard_layer" style="display: none;"></div>
          </div>`
}

export default plugin;
