import axios from 'axios'
import autobind from 'autobind-decorator'

class Dialog {
  constructor(editor) {
    this.editor = editor
    this.listUrl = editor.settings.google_photos.list_url
  }

  @autobind
  onReceiveMessage(e) {
    // TDOO
    console.log("receive message", e)
    const message = e.data
    if (message == 'finishGoogleAuth') {
      this.$authView.hide()
      this.firstFetch()
    } else if (message == 'failedGoogleAuth') {
      this.$authView.hide()
    }
  }

  @autobind
  onOpen() {
    const $ = this.win.$
    this.$mainView = $('.main-view')
    this.$body = this.$mainView.find('.body')
    this.$authView = $('.auth-view')
    this.$loadingView = $('.loading-view')
    this.$errorView = $('.error-view')
    this.$btnAuth = this.$authView.find('.btn-auth')
    
    
    this.$btnAuth.on('click', e => {
      if (this.authUrl) {
        window.open(this.authUrl, 'auth', 'width=600px,height=480px')
      }
    })
    this.$body.on('click', e => {
      e.preventDefault()
      const $img = $(e.target)
      if ($img.attr('class') == 'photos-image') {
        this.editor.insertContent(`<img src='${$img.attr('data-url')}'>`)
      }
    })

    window.addEventListener("message", this.onReceiveMessage, false)
    this.firstFetch()
  }

  @autobind
  onClose() {
    window.removeEventListener("message", this.onReceiveMessage, false)
  }

  open() {
    this.makeWindow()
    this.onOpen()
  }

  async firstFetch() {
    this.$loadingView.show()
    axios.get(this.listUrl)
      .then(res => res.data)
      .then(images => {
        images.map(image => {
          this.$body.append(`<span class='photos-image-wrap'><img src='${image.thumbnail}' data-url='${image.url}' data-title='${image.title}' class='photos-image'></span>`)
        })
        
        this.$loadingView.hide()
      })
      .catch(err => {
        if (err.response.status == 403) {
          this.authUrl = err.response.data.authUrl
          this.$authView.show()
        } else {
          this.$errorView.show()
        }
        this.$loadingView.hide()
      })
  }

  makeWindow() {
    this.win = this.editor.windowManager.open({
      title: '구글 Photos 이미지 불러오기',
      items: [{
        type: 'container',
        html: `<div class="keditor-dialog" style="width:800px; height:540px">
          <div class='view main-view'>
            <div class="body">
            </div>
            <div class="footer">
              <button type="reset" class="btn btn-cancel">취소</button>
              <button type="submit" class="btn btn-default">확인</button>
            </div>
          </div>
          <div class='view layer-view auth-view' style='display:none'><button class="btn btn-auth" id="btn-google-auth">Google 인증</button></div>
          <div class='view layer-view error-view' style='display:none'><span>오류가 발생했습니다.</span><button class="btn">새로고침</button></div>
          <div class='view layer-view loading-view' style='display:none'><span class='loading'>로딩 중</span></div>
        </div>`
      }],
      buttons: [],
      onClose: this.onClose
    })
    this.win.statusbar.remove()
  }
}

export default Dialog
