const plugin = editor => {
  const updateSelection = function(editor) {
    return function(e) {
      const hrStyleType = `<hr contenteditable="false" data-ke-style="${editor.selection
        .getNode()
        .getAttribute('data-ke-style')}" />`
      e.control.items().each(function(ctrl) {
        ctrl.active(ctrl.settings.value === hrStyleType)
      })
    }
  }

  const createFormatList = function(editor) {
    const createMenu = function(formats: Array<any>) {
      const menu = []

      if (!formats) {
        return
      }

      formats.forEach(function(format) {
        const menuItem: any = {
          // text: format.title,
          icon: format.icon,
        }

        let htmlString = ''
        for (const key in format.attributes) {
          htmlString += `<hr data-ke-type="hr" contenteditable="false" ${key}="${format.attributes[key]}" />`
        }
        menuItem.html = htmlString

        menu.push(menuItem)
      })

      return menu
    }

    return createMenu(editor.settings.hrList)
  }

  const registerButtons = function(editor, list) {
    const items = []
    const align = editor.settings.hrList_align || 'center'

    list.forEach(function(item) {
      items.push({
        text: item.text,
        value: item.html,
        icon: item.icon,
        classes: 'hr',
      })
    })
    editor.addButton('hrList', {
      type: 'simplemenubutton',
      icon: 'hrList',
      tooltip: 'Horizontal line',
      fixedWidth: true,
      menu: {
        items,
        onPostRender(e) {
          e.control.fixed(true)
        },
      },
      onPostRender(e) {
        const ctrl = e.control
        editor.on('NodeChange', function(e) {
          const parents: Array<Node> = e.parents
          if (parents && parents[parents.length - 1]) {
            ctrl.active(parents[parents.length - 1].nodeName === 'HR')
          }
        })
      },
      onshow: updateSelection(editor),
      onselect(e) {
        if (e.control) {
          const selectNode: Node = editor.selection.getNode()
          const curNode = editor.selection.getNode()
          if (curNode.nodeName === 'HR') {
            curNode.outerHTML = e.control.value()
            editor.undoManager.add()
          } else {
            if (['IMG', 'FIGCAPTION'].indexOf(selectNode.nodeName) == -1) {
              editor.execCommand('mceInsertContent', false, e.control.value())
            }
          }
        }
      },
      onShowMenu() {
        if (align === 'center') {
          this.menu.moveRel(this.getEl(), ['bc-tc', 'tc-bc'])
        }
      },
    })
  }

  const formatList = createFormatList(editor)
  registerButtons(editor, formatList)
}

export default plugin
