import HeadingDataModel from './components/Heading'
import ParagraphDataModel from './components/Paragraph'
import TextDataModel from './components/Text'
import LinkDataModel from './components/Link'
import ImageDataModel from './components/Image'
import MapDataModel from './components/Map'
import VideoDataModel from './components/Video'
import ListsDataModel from './components/Lists'
import HrDataModel from './components/Hr'
import BlockquoteDataModel from './components/Blockquote'
import TableDataModel from './components/Table'

import ItemDataType from './model/ItemDataType.json'

class DataModel {
  rootTreeContent: any
  contents: any[]
  images: any[]
  videos: any[]
  maps: any[]
  constructor(tree) {
    this.rootTreeContent = tree
    this.contents = []
    this.images = []
    this.videos = []
    this.maps = []
  }

  getKeType(obj) {
    if (obj && obj.attributes && obj.attributes.map) {
      return obj.attributes.map['data-ke-type']
    } else {
      return false
    }
  }

  makeData(treeObject, parentArray) {
    let itemObject: any = {},
      type,
      itemAttribute: any = {},
      stopChildPropagation = false
    const keType = this.getKeType(treeObject)
    const treeNodeName = treeObject.name

    if (ItemDataType.paragraph.nodeName.includes(treeNodeName)) {
      type = ItemDataType.paragraph.typeName
      ParagraphDataModel.register(treeObject, itemAttribute)
    } else if (ItemDataType.heading.nodeName.includes(treeNodeName)) {
      type = ItemDataType.heading.typeName
      HeadingDataModel.register(treeObject, itemAttribute)
    } else if (ItemDataType.text.nodeName.includes(treeNodeName) && treeObject.parent.name !== 'figure') {
      // figure node에는 바로 #text가 오지 않도록 하기
      type = ItemDataType.text.typeName
      TextDataModel.register(treeObject, itemAttribute)
    } else if (ItemDataType.link.nodeName.includes(treeNodeName)) {
      type = ItemDataType.link.typeName
      LinkDataModel.register(treeObject, itemAttribute)
    } else if (ItemDataType.hr.nodeName.includes(treeNodeName)) {
      type = ItemDataType.hr.typeName
      HrDataModel.register(treeObject, itemAttribute)
    } else if (keType === ItemDataType.keTypeImage.keType) {
      type = ItemDataType.keTypeImage.typeName
    } else if (keType === ItemDataType.keTypeGirdImage.keType) {
      type = ItemDataType.keTypeGirdImage.typeName
    } else if (ItemDataType.image.nodeName.includes(treeNodeName)) {
      type = ItemDataType.image.typeName
      ImageDataModel.register(treeObject, itemAttribute)
      this.images.push(itemAttribute)
    } else if (ItemDataType.caption.nodeName.includes(treeNodeName)) {
      type = ItemDataType.caption.typeName
    } else if (ItemDataType.codeBlockWrap.nodeName.includes(treeNodeName)) {
      if (treeObject.firstChild && ItemDataType.codeBlockWrap.childNodeName.includes(treeObject.firstChild.name)) {
        type = ItemDataType.codeBlockWrap.typeName
        itemAttribute.text = treeObject.firstChild.firstChild.value || ''
        stopChildPropagation = true
      }
    } else if (keType === ItemDataType.keTypeMap.keType) {
      type = ItemDataType.keTypeMap.typeName
      MapDataModel.register(treeObject, itemAttribute)
      stopChildPropagation = true
      this.maps.push(itemAttribute)
    } else if (keType === ItemDataType.keTypeImage.keType) {
      type = ItemDataType.keTypeImage.typeName
    } else if (keType === ItemDataType.keTypeVideo.keType) {
      type = ItemDataType.keTypeVideo.typeName
      VideoDataModel.register(treeObject, itemAttribute)
      this.videos.push(itemAttribute)
    } else if (ItemDataType.lists.nodeName.includes(treeNodeName)) {
      type = ItemDataType.lists.typeName
      ListsDataModel.register(treeObject, itemAttribute)
    } else if (ItemDataType.list.nodeName.includes(treeNodeName)) {
      type = ItemDataType.list.typeName
    } else if (ItemDataType.br.nodeName.includes(treeNodeName)) {
      type = ItemDataType.br.typeName
    } else if (ItemDataType.blockquote.nodeName.includes(treeNodeName)) {
      type = ItemDataType.blockquote.typeName
      BlockquoteDataModel.register(treeObject, itemAttribute)
    } else if (keType === ItemDataType.keTypeMoreLess.keType) {
      type = ItemDataType.keTypeMoreLess.typeName
    } else if (ItemDataType.table.nodeName.includes(treeNodeName)) {
      type = ItemDataType.table.typeName
      TableDataModel.register(treeObject, itemAttribute)
    }

    if (type) {
      if (Object.keys(itemAttribute).length != 0) {
        itemObject.attribute = itemAttribute
      }
      itemObject.type = type
      parentArray.push(itemObject)
    }

    if (!stopChildPropagation && treeObject.firstChild) {
      if (!itemObject.child) {
        itemObject.child = []
      }
      this.makeData(treeObject.firstChild, itemObject.child)
    }
    if (treeObject.next) {
      this.makeData(treeObject.next, parentArray)
    }
  }

  getData() {
    this.makeData(this.rootTreeContent.firstChild, this.contents)

    return {
      contents: this.contents,
      images: this.images,
      videos: this.videos,
      maps: this.maps,
    }
  }
}

export default DataModel
