declare global {
  interface Window {
    tinymce: any
  }
}

class Caption {
  editor: any
  selectedNode: any
  enableCaptionTypeArry: string[] = ['image', 'imageGrid', 'imageSlide', 'video']
  constructor(editor) {
    this.editor = editor
    this.selectedNode = undefined
    this.init()
  }

  init() {
    this.editor.on('NodeChange', e => {
      if (e.selectionChange) {
        const $figureElement = this.editor.$(e.element).closest('figure')
        const keType = $figureElement.attr('data-ke-type')
        if (this.enableCaptionTypeArry.includes(keType)) {
          this.makeCaption($figureElement)
        }
        window.setTimeout(() => {
          this.removeCaption()
        }, 1)
      }
    })

    this.editor.on('Keyup', e => {
      if (e.keyCode == 8 || e.keyCode == 46) {
        const selectedNode = this.editor.selection.getNode()
        if (selectedNode.nodeName == 'FIGCAPTION' && selectedNode.textContent == '') {
          selectedNode.innerHTML == ''
          if (selectedNode.firstChild) {
            selectedNode.removeChild(selectedNode.firstChild)
          }
        }
      }
    })
  }

  makeCaption($figureElement) {
    this.editor.undoManager.ignore(() => {
      let figcaptionElement: HTMLElement
      const $ = this.editor.$,
        dom = this.editor.dom
      const itemStyle: String = $figureElement.attr('data-ke-style') || ''
      if (itemStyle.split('float').length == 1) {
        const $figcaption = $figureElement.find('figcaption')
        if ($figcaption.length > 0) {
          if ($figureElement.attr('contenteditable') === 'false') {
            $figcaption.attr('contenteditable', 'true')
          }
          $figcaption.show()
        } else {
          figcaptionElement = dom.create('figcaption')
          if ($figureElement.attr('contenteditable') === 'false') {
            figcaptionElement.setAttribute('contenteditable', 'true')
          }
          $figureElement.append(figcaptionElement)
        }
      }
    })
  }

  removeCaption() {
    const $ = this.editor.$
    const selectionElement = this.editor.selection.getNode()
    const $captions = $('figcaption:empty', this.editor.getBody())
    for (let i = 0, l = $captions.length; i < l; i++) {
      if (
        selectionElement != $captions[i] &&
        selectionElement.parentNode.lastChild != $captions[i] &&
        selectionElement.lastChild != $captions[i]
      ) {
        $captions[i].style.display = 'none'
      }
    }
  }
}

export default Caption
