import AlignFloatAction from './action/AlignFloatAction'
import AlignFloatButtons from './ui/FloatButtons'
import initPhotos from '../components/initPhotos'
import representButton from '../components/ui/RepresentButton'
import resizeButton from '../components/ui/ResizeButton'

enum ActionCommandName {
  alignLeft = 'kImageAlignLeft',
  alignCenter = 'kImageAlignCenter',
  alignRight = 'kImageAlignRight',
  floatLeft = 'kImageFloatLeft',
  floatRight = 'kImageFloatRight',
}

class ContextToolbar {
  editor: any
  toolbarButtonArray: any[]
  toolbar: any[]
  toolbarArray: string[]
  AlignFloatAction: AlignFloatAction
  constructor(editor) {
    this.editor = editor
    this.toolbarButtonArray = []
    const toolbarString: String = (editor.settings.kImage && editor.settings.kImage.toolbar) || ''
    this.toolbarArray = toolbarString.split(' ')
    this.AlignFloatAction = new AlignFloatAction(this.editor)
  }

  add(buttonName) {
    this.toolbarButtonArray.push(buttonName)
  }

  predicate(element) {
    return this.editor.dom.is(element, 'figure[data-ke-type="image"], figure[data-ke-type="image"] img')
  }

  predicateForGrid(element) {
    return this.editor.dom.is(element, 'figure[data-ke-type="imageGrid"] img')
  }

  register(photosSucessCallback, photosCancelCallback, editingImageId) {
    this.toolbarArray.forEach((value, index) => {
      if (ActionCommandName[value]) {
        this.AlignFloatAction.register(value, ActionCommandName[value])
        AlignFloatButtons.register(this.editor, value)
        this.add(ActionCommandName[value])
      } else if (value === 'edit') {
        initPhotos(this.editor, photosSucessCallback, editingImageId, photosCancelCallback)
        this.add('kImageEdit')
      } else if (value === 'representative') {
        representButton.register(this.editor)
      } else if (value === 'resize') {
        resizeButton.register(this.editor)
        this.add('kImageResize')
      } else if (value === '|') {
        this.add('|')
      }
    })

    if (this.toolbarButtonArray.indexOf('kImageEdit') > -1) {
      this.editor.addContextToolbar(this.predicateForGrid.bind(this), 'kImageEdit')
    }

    this.editor.addContextToolbar(this.predicate.bind(this), this.toolbarButtonArray.join(' '))
  }
}

export default ContextToolbar
