const register = editor => {
  const imageItemSelector = '[data-ke-type="image"], [data-ke-type="imageGrid"]'
  editor.on('KeyDown', e => {
    const selectedNode = editor.selection.getNode()
    const $selectedNode = editor.$(selectedNode)
    let $imageFigure, $figcaption
    if (!selectedNode) {
      return false
    }
    if (e.keyCode == 8 || e.keyCode == 46) {
      if (selectedNode.nodeName == 'IMG') {
        $imageFigure = $selectedNode.closest(imageItemSelector)
        if ($imageFigure[0] && $imageFigure.attr('data-ke-type') == 'image') {
          e.preventDefault()
          e.stopPropagation()
          const imageFigureNextSibling = $imageFigure[0].nextSibling
          $imageFigure.remove()
          if (imageFigureNextSibling) {
            setCursorLocationFirst(imageFigureNextSibling)
          }
          editor.undoManager.add()
        } else if ($imageFigure.attr('data-ke-type') == 'imageGrid') {
          e.preventDefault()
          e.stopPropagation()
          let $previousImageNode = $selectedNode.prev() || $imageFigure.find('img')
          $selectedNode.remove()
          editor.execCommand('updateGridForImageDelete')
          editor.selection.select($previousImageNode[0])
        }
      } else if (
        selectedNode.nodeName == 'FIGCAPTION' &&
        editor.selection.getSel().focusOffset == 0 &&
        editor.selection.getSel().anchorOffset == 0
      ) {
        e.preventDefault()
        e.stopPropagation()
      }
    } else if (e.keyCode == 13) {
      if (selectedNode.nodeName == 'IMG') {
        e.preventDefault()
        e.stopPropagation()
        $imageFigure = $selectedNode.closest(imageItemSelector)
        $figcaption = editor.$('figcaption', $imageFigure[0])
        if ($figcaption[0] && $figcaption[0].style.display != 'none') {
          setCursorLocationFirst($figcaption[0])
        } else {
          $imageFigure = $selectedNode.closest(imageItemSelector)
          appendAfterNewParagraph($imageFigure)
        }
      }
    }
  })

  const appendAfterNewParagraph = targetElement => {
    const $targetElement = editor.$(targetElement)
    if (!$targetElement.next()[0] || $targetElement.next()[0].nodeName != 'P') {
      $targetElement.after('<p><br></p>')
    }
    setCursorLocationFirst($targetElement.next()[0])
  }

  const setCursorLocationFirst = targetElement => {
    const range = editor.selection.getRng()
    range.setStart(targetElement, 0)
    range.setEnd(targetElement, 0)
    editor.selection.setCursorLocation(targetElement)
    editor.selection.getSel().removeAllRanges()
    editor.selection.getSel().addRange(range)
  }

  const appendBeforeNewParagraph = targetElement => {
    const $targetElement = editor.$(targetElement)
    $targetElement.before('<p><br></p>')
    setCursorLocationFirst($targetElement.prev()[0])
  }
}

export default {
  register,
}
