import DOMUtils from 'tinymce/core/api/dom/DOMUtils'

class Panel {
  editor: any
  panel: any
  widthInput: HTMLInputElement
  contentSize: number
  widthInputId: String
  MinWidth: Number = 100
  $widthInput: any
  constructor(editor: any, widthInputId: String) {
    this.editor = editor
    this.widthInputId = widthInputId

    this.hidePanel = this.hidePanel.bind(this)
  }

  init(panel: any) {
    this.panel = panel
    this.$widthInput = panel.$('#' + this.widthInputId, panel.$el[0])
    this.contentSize = this.editor.getBody().offsetWidth
    if (this.editor.settings.autoresize_overflow_padding) {
      this.contentSize -= this.editor.settings.autoresize_overflow_padding * 2
    }
    this.editor.on('ObjectResizeStart', function() {
      panel.hide()
    })

    let scrollContainer
    if (this.editor.settings.scroll_container_id) {
      scrollContainer = document.getElementById(this.editor.settings.scroll_container_id)
    } else {
      scrollContainer = this.editor.selection.getScrollContainer() || this.editor.getWin()
    }

    DOMUtils.DOM.bind(scrollContainer, 'scroll', this.hidePanel)

    this.editor.on('remove', function() {
      DOMUtils.DOM.unbind(scrollContainer, 'scroll', this.hidePanel)
    })
  }

  hidePanel() {
    this.panel.hide()
  }

  getImageElem(): HTMLImageElement {
    const selectNode: HTMLImageElement = this.editor.selection.getNode()
    if (selectNode.nodeName === 'IMG') {
      return selectNode
    } else {
      const $selectNode: any = this.editor.$(selectNode)
      const $figureClosestNode = $selectNode.closest('figure')
      if ($figureClosestNode && $figureClosestNode[0]) {
        return $figureClosestNode[0].firstElementChild
      }
    }
  }

  updateMaxSize() {
    this.updateImageWidth(this.contentSize)
    this.panel.hide()
  }

  updateImageWidth(width) {
    width = parseInt(width)
    if (!width) {
      return false
    }
    const imageElme = this.getImageElem()
    width = width > this.contentSize ? this.contentSize : width
    width = width < this.MinWidth ? this.MinWidth : width
    imageElme.setAttribute('width', width)
    window.setTimeout(() => {
      this.editor.execCommand('mceAutoResize')
      this.editor.execCommand('updateRepresentButton', imageElme)
    })
  }

  show() {
    const imageElem = this.getImageElem()
    if (imageElem) {
      let width: number = Number(imageElem.getAttribute('width'))
      if (width === 0) {
        width = Number(imageElem.offsetWidth)
      }
      this.$widthInput[0].value = width > this.contentSize ? this.contentSize : width
    }
  }
}

export default Panel
