declare global {
  interface Window {
    tinymce: any
  }
}

const register = editor => {
  let representButton, representTartgetImage, callback, scrollContainer, scrollRoot
  const editorDocumentRoot = editor.dom.getRoot()
  let representImage = editor.$('figure[data-ke-type^="image"] img')[0]

  if (editor.settings.kImage) {
    if (editor.settings.kImage.representative_image_callback) {
      callback = editor.settings.kImage.representative_image_callback
    }
  }

  if (editor.settings.scroll_container_id) {
    scrollContainer = document.getElementById(editor.settings.scroll_container_id)
  }

  if (scrollContainer) {
    scrollRoot = scrollContainer
  } else {
    scrollContainer = editor.selection.getScrollContainer() || editor.getWin()
    scrollRoot = editorDocumentRoot.ownerDocument.documentElement || editorDocumentRoot
  }

  const getRepresentImage = () => {
    return representTartgetImage
  }

  const setPositionAndStyle = buttonNode => {
    const isMobileFullscreen = !!document.querySelector('[data-ephox-mobile-fullscreen-style]')
    const imageRect = getElementRect(representTartgetImage, isMobileFullscreen)
    const rectX = imageRect.x + imageRect.w - 64
    const rectY = imageRect.y + 5
    buttonNode.style.left = rectX + 'px'
    buttonNode.style.top = rectY + 'px'
    if (representTartgetImage == representImage) {
      representButton.classList.add('active')
    } else {
      representButton.classList.remove('active')
    }
    buttonNode.keditor_rect = { x: rectX, y: rectY }

    const scrollTop = editorDocumentRoot.ownerDocument.documentElement.scrollTop || editorDocumentRoot.scrollTop
    buttonNode.keditor_scroll = { y: scrollTop }
  }

  const showButton = imageNode => {
    const isMobileFullscreen = !!document.querySelector('[data-ephox-mobile-fullscreen-style]')
    let buttonRoot = document.body
    if (isMobileFullscreen) {
      buttonRoot = editor.getBody()
    }
    representTartgetImage = imageNode

    if (representButton) {
      setPositionAndStyle(representButton)
      representButton.style.display = 'block'
    } else {
      representButton = getButton()
      setPositionAndStyle(representButton)

      editor.$(representButton).on('click touchstart', setRepresentImage)
      buttonRoot.appendChild(representButton)
    }
  }

  const showButtonDelay = imageNode => {
    window.tinymce.util.Delay.setTimeout(() => {
      showButton(imageNode)
    }, 1)
  }

  const setRepresentImage = e => {
    const imageNode = getRepresentImage()
    representImage = imageNode
    representButton.classList.add('active')
    const data = {
      src: imageNode.getAttribute('src'),
    }
    if (callback) {
      callback(data)
    } else {
      console.log(data)
    }
  }

  const hideButton = () => {
    if (representButton) {
      representButton.style.display = 'none'
    }
  }

  const getElementRect = (elem, isMobile) => {
    let pos, targetRect, root

    targetRect = editor.dom.getRect(elem)
    root = editor.dom.getRoot()

    if (!isMobile) {
      pos = window.tinymce.DOM.getPos(editor.getContentAreaContainer())
      targetRect.x += pos.x
      targetRect.y += pos.y
    }

    if (root.nodeName === 'BODY') {
      targetRect.x -= root.ownerDocument.documentElement.scrollLeft || root.scrollLeft
      targetRect.y -= root.ownerDocument.documentElement.scrollTop || root.scrollTop
    }

    return targetRect
  }

  const repositionButton = () => {
    if (representButton && representButton.style.display != 'none') {
      setPositionAndStyle(representButton)
    }
  }

  const getButton = () => {
    return editor.dom.create('div', {
      contenteditable: 'false',
      tabindex: '-1',
      'data-mce-bogus': 'all',
      class: 'mce-represent-image-btn',
    })
  }

  editor.addCommand('updateRepresentButton', function(imageNode) {
    if (imageNode) {
      showButtonDelay(imageNode)
    }
  })

  editor.on('nodeChange', e => {
    if (e.selectionChange) {
      if (editor.dom.is(e.element, 'figure[data-ke-type^="image"] img')) {
        showButtonDelay(e.element)
      } else if (e.element.className !== 'mce-represent-image-btn') {
        hideButton()
      }
    }
  })

  editor.on('blur', e => {
    const isMobileFullscreen = !!document.querySelector('[data-ephox-mobile-fullscreen-style]')
    if (!isMobileFullscreen) {
      hideButton()
    }
  })

  editor.on('focus', e => {
    const seletNode = editor.selection.getNode()
    if (editor.dom.is(seletNode, 'figure[data-ke-type^="image"] img')) {
      showButtonDelay(seletNode)
    }
  })

  editor.on('ObjectResized', e => {
    if (editor.dom.is(e.target, 'figure[data-ke-type^="image"] img')) {
      showButtonDelay(e.target)
    }
  })

  editor.on('ResizeEditor ResizeWindow', repositionButton)
  editor.$(scrollContainer).on('scroll', repositionButton)
}

export default {
  register,
}
