import Tools from 'tinymce/core/api/util/Tools'
import NodeType from '../core/NodeType'
import ListUtils from '../core/ListUtils'
import ListStyles from './ListStyles'

const findIndex = function(list, predicate) {
  for (let index = 0; index < list.length; index++) {
    const element = list[index]

    if (predicate(element)) {
      return index
    }
  }
  return -1
}

const checkNode = function(parentsList) {
  const tableCellIndex = findIndex(parentsList, NodeType.isTableCellNode)
  const parents = tableCellIndex !== -1 ? parentsList.slice(0, tableCellIndex) : parentsList
  const lists = Tools.grep(parents, NodeType.isListNode)

  return lists.length > 0 && (lists[0].nodeName === 'OL' || lists[0].nodeName === 'UL')
}

const listState = function(editor) {
  return function(e) {
    const ctrl = e.control

    editor.on('NodeChange', function(e) {
      ctrl.active(checkNode(e.parents))
    })
  }
}

const updateSelection = function(editor) {
  return function(e) {
    const listStyleType = ListUtils.getSelectedStyleType(editor)
    e.control.items().each(function(ctrl) {
      ctrl.active(ctrl.settings.style === listStyleType)
    })
  }
}

const applyListFormat = function(editor, listName, styleValue) {
  const cmd = listName === 'UL' ? 'InsertUnorderedList' : 'InsertOrderedList'
  editor.execCommand(cmd, false, styleValue === false ? null : { 'list-style-type': styleValue })
}

const addSplitButton = function(editor, id, tooltip, setting) {
  const styles = setting.list;
  const align = setting.align || 'left';

  if (!styles) {
    return;
  }

  editor.addButton(id, {
    active: false,
    type: 'simplemenubutton',
    tooltip,
    menu: {
      items: ListStyles.toMenuItems(styles),
      onPostRender(e) {
        e.control.fixed(true)
      },
    },
    onPostRender: listState(editor),
    onshow: updateSelection(editor),
    onselect(e) {
      const selectNode: Node = editor.selection.getNode()
      if (['IMG', 'FIGURE', 'FIGCAPTION'].indexOf(selectNode.nodeName) == -1) {
        applyListFormat(editor, e.control.settings.nodeType, e.control.settings.style)
      }
    },
    onhide(e) {
      if (checkNode(editor.dom.getParents(editor.selection.getNode(), '*', editor.getBody()))) {
        e.control.parents('*')[0].classes.toggle('active', true)
      }
    },
    onShowMenu() {
      if (align === 'center') {
        this.menu.moveRel(this.getEl(), ['bc-tc', 'tc-bc']);
      }
    }
  })
}

const register = function(editor) {
  const hasPlugin = function(editor, plugin) {
    const plugins = editor.settings.plugins ? editor.settings.plugins : ''
    return Tools.inArray(plugins.split(/[ ,]/), plugin) !== -1
  }

  const settingListData = editor.settings.kList ? editor.settings.kList.list : null
  if (editor.settings.kList) {
    addSplitButton(editor, 'kList', 'List', editor.settings.kList)
  }
}

export default {
  register,
}
