import DOMUtils from 'tinymce/core/api/dom/DOMUtils';

const DOM = DOMUtils.DOM;

class Panel {
  editor: any
  panel: any
  widthInput: HTMLInputElement
  contentSize: number
  constructor(editor: any) {
    this.editor = editor

    this.hidePanel = this.hidePanel.bind(this);
  }

  init(panel: any) {
    this.panel = panel
    const panelNode = panel.$el[0]
    this.addEvent(panelNode)

    let scrollContainer;
    if (this.editor.settings.scroll_container_id) {
      scrollContainer = document.getElementById(this.editor.settings.scroll_container_id);
    } else {
      scrollContainer = this.editor.selection.getScrollContainer() || this.editor.getWin();  
    }
    
    DOM.bind(scrollContainer, 'scroll', this.hidePanel);

    this.editor.on('remove', function () {
      DOM.unbind(scrollContainer, 'scroll', this.hidePanel);
    });
  }

  hidePanel() {
    this.panel.hide();
  }

  getPanelHtml() {
    let html = `<div class="keditor-tablepreset-panel">`
    for (let i = 0, l = 16; i < l; i++) {
      html += `<button class="ico-tablepreset" data-style="${i}"></button>`
    }
    return html + `</div>`
  }

  private addEvent(panelNode: Node) {
    this.editor.$('button', panelNode).on('click', this.applyStyle.bind(this))
  }

  private applyStyle(e) {
    e.preventDefault()
    const applyStyle: String = e.target.dataset.style
    const $tableElem = this.editor.$(this.editor.selection.getNode()).closest('table')
    if (applyStyle === '0') {
      $tableElem.removeAttr('data-ke-style')
    } else {
      $tableElem.attr('data-ke-style', 'style' + applyStyle)
    }

    this.hidePanel();
  }
}

export default Panel
