/**
 * Plugin.js
 *
 * Released under LGPL License.
 * Copyright (c) 1999-2017 Ephox Corp. All rights reserved
 *
 * License: http://www.tinymce.com/license
 * Contributing: http://www.tinymce.com/contributing
 */

import { Cell } from '@ephox/katamari'
import Api from './api/Api'
import Settings from './api/Settings'
import Keyboard from './core/Keyboard'

declare global {
  interface Window {
    tinymce: any
  }
}
const tinymce = window.tinymce || {}

tinymce.PluginManager.add('kTextpattern', function(editor) {
  const patternsState = Cell(Settings.getPatterns(editor.settings))

  Keyboard.bind(editor, patternsState)

  editor.addCommand('bindKtextpattern', function() {
    Keyboard.bind(editor, patternsState)
  })

  editor.addCommand('unbindKtextpattern', function() {
    Keyboard.unbind()
  })

  const items = [
    {
      text: 'On MD',
      value: 'bindKtextpattern',
      icon: 'code',
    },
    {
      text: 'Off MD',
      value: 'unbindKtextpattern',
      icon: 'code',
    },
  ]

  editor.addButton('kTextpattern', {
    type: 'menubutton',
    icon: 'code',
    tooltip: 'markdown',
    menu: {
      items,
      onPostRender(e) {
        e.control.fixed(true)
      },
    },
    onselect(e) {
      if (e.control && e.control.value()) {
        editor.execCommand(e.control.value())
      }
    },
  })

  return Api.get(patternsState)
})

export default function() {}
