/**
 * Keyboard.js
 *
 * Released under LGPL License.
 * Copyright (c) 1999-2017 Ephox Corp. All rights reserved
 *
 * License: http://www.tinymce.com/license
 * Contributing: http://www.tinymce.com/contributing
 */

import Delay from 'tinymce/core/api/util/Delay'
import VK from 'tinymce/core/api/util/VK'
import KeyHandler from './KeyHandler'

const handler = {
  keydown: null,
  keyup: null,
  keypress: null,
}
const bind = function(editor, patternsState) {
  if (handler.keydown) {
    return false
  }
  const charCodes = [',', '.', ';', ':', '!', '?']
  const keyCodes = [32]

  handler.keydown = editor.on(
    'keydown',
    function(e) {
      if (e.keyCode === 13 && !VK.modifierPressed(e)) {
        KeyHandler.handleEnter(editor, patternsState.get())
      }
    },
    true
  )

  handler.keyup = editor.on('keyup', function(e) {
    if (KeyHandler.checkKeyCode(keyCodes, e)) {
      KeyHandler.handleInlineKey(editor, patternsState.get())
    }
  })

  handler.keypress = editor.on('keypress', function(e) {
    if (KeyHandler.checkCharCode(charCodes, e)) {
      Delay.setEditorTimeout(editor, function() {
        KeyHandler.handleInlineKey(editor, patternsState.get())
      })
    }
  })
}

const unbind = function() {
  if (!handler.keydown) {
    return false
  }
  for (const key in handler) {
    handler[key].off()
    handler[key] = null
  }
}

export default {
  bind,
  unbind,
}
