const makeLink = editor => {
  const curNode = editor.selection.getNode()

  for (let i = 0; i < curNode.childNodes.length; i++) {
    const child = curNode.childNodes[i]
    if (child.nodeName !== '#text' && child.getAttribute('data-mce-type') === 'bookmark') {
      editor.dom.remove(child)
    }
  }

  const bm = editor.selection.getBookmark()
  curNode.innerHTML = curNode.innerHTML
  let str = curNode.innerHTML

  str = str.replace(/(<span.*?data-mce-type="bookmark".*?><\/span>($|<br data-mce-bogus="1">))/gi, (...args) => {
    return `&nbsp;${args[1]}`
  })

  curNode.innerHTML = str
  editor.undoManager.add()

  str = curNode.innerHTML
  str = str.replace(
    /([\s]{1}|^)((https?:\/\/)?[www\.]?[-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,6}\b[-a-zA-Z0-9@:%_\+.~#?&\/\/=\(\)\[\]!$'*,;]*?)/gi,
    (...args) => {
      const hasLeadingSpace = !!args[1]
      const url = args[2]
      let href = url
      const firstNbspIndex = url.indexOf('&nbsp;')
      let result = hasLeadingSpace ? ' ' : ''

      const webProtocolRegExp = /https?:\/\//gi
      if (!webProtocolRegExp.test(url)) {
        href = 'http://' + url
      }

      if (firstNbspIndex > -1) {
        result += `<a href="${href.slice(0, firstNbspIndex)}">${url.slice(0, firstNbspIndex)}</a>&nbsp;`
      } else {
        result += `<a href="${href}">${url}</a>`
      }

      return result
    }
  )

  curNode.innerHTML = str
  const bmElement = editor.dom.select('span[data-mce-type=bookmark]')[0]

  editor.selection.moveToBookmark(bm)
  editor.undoManager.add()
}

const autoLinkInit = editor => {
  editor.on('keydown', e => {
    const node = editor.selection.getNode()
    if (!editor.dom.is(node, 'p, h1, h2, h3, b, i, strong, em, span, s, td')) {
      return
    }

    const urlRegExp = /([\s]{1}|^)((https?:\/\/)?[www\.]?[-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,6}\b[-a-zA-Z0-9@:%_\+.~#?&\/\/=\(\)\[\]!$'*,;]*)/g
    const autoUrlRegExp = /([\s]{1}|^)((www\.)|(m\.)|(http:\/\/)|(https:\/\/))[-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,6}\b[-a-zA-Z0-9@:%_\+.~#?&\/\/=\(\)\[\]!$'*,;]*/g

    if (e.keyCode === 32 || (e.code === 'Space' && e.key === 'Process')) {
      if (urlRegExp.test(node.innerHTML) && autoUrlRegExp.test(node.innerHTML)) {
        makeLink(editor)
        e.preventDefault()
      }
    } else if (e.keyCode === 13) {
      if (urlRegExp.test(node.innerHTML) && autoUrlRegExp.test(node.innerHTML)) {
        makeLink(editor)
      }
    }
  })
}

export { autoLinkInit }
