import LinkDialog from './ui/dialog/LinkDialog'
import generatePanelHtml from './ui/panel/generatePanelHtml'
import LinkPanel from './ui/panel/LinkPanel'
import { autoLinkInit } from './components/autoLink'

export default editor => {
  const Delay = tinymce.util.Tools.resolve('tinymce.util.Delay')
  const $ = editor.$
  const settings = editor.settings.klink
  const isPanel = settings && settings.is_panel
  const align = (settings && settings.align) || 'left'

  if (settings && settings.auto_link) {
    autoLinkInit(editor)
  }

  const onPostRender = editor => {
    return e => {
      const control = e.control
      const disabledStateSelector = control.settings.disabledStateSelector
      editor.shortcuts.add('meta+k', '', () => {
        if (!control.disabled()) {
          control.showPanel()
        }
      })

      editor.on(
        'nodeChange mouseup keyUp',
        Delay.throttle(() => {
          if (!disabledStateSelector) {
            return
          }

          let disabled = false
          const selectionNode = editor.selection.getNode()
          if (editor.dom.is(selectionNode, disabledStateSelector)) {
            disabled = true
          }

          control.disabled(disabled)
        }, 0)
      )
    }
  }

  if (isPanel) {
    const linkPanel = new LinkPanel(editor)

    editor.addButton('klink', {
      icon: 'link',
      type: 'panelbutton',
      tooltip: 'Insert/edit link',
      popoverAlign: align === 'center' ? ['bc-tc'] : ['bc-tl'],
      stateSelector: 'a',
      disabledStateSelector: 'img,figure,iframe,div,figcaption,table,pre',
      panel: {
        role: 'dialog',
        html: generatePanelHtml(),
        onPostRender() {
          this.fixed(true)
        },
      },
      onPostRender: onPostRender(editor),
      onshow() {
        linkPanel.open(this.panel)
      },
      onhide() {
        linkPanel.removeEvent()
        editor.focus()
      },
    })
  } else {
    const linkDialog = new LinkDialog(editor)

    editor.addCommand('klink', () => {
      linkDialog.open()
    })
    editor.addButton('klink', {
      cmd: 'klink',
      icon: 'link',
      tooltip: 'Insert/edit link',
      stateSelector: 'a',
    })
    editor.shortcuts.add('meta+k', 'open link', 'klink')
  }
}
