import autobind from 'autobind-decorator'
import * as LinkUtils from '../../utils/LinkUtils'

class LinkPanel {
  constructor(editor, panel) {
    this.editor = editor
  }

  open(panel) {
    this.rng = this.editor.selection.getRng()
    this.makePanelItem(panel)
    this.setEvent()
    this.setData()
    this.checkStatus()
    this.$url[0].select()
  }

  makePanelItem(panel) {
    const panelDocument = window.parent.document
    const editor = this.editor

    this.panel = panel
    this.$btn = editor.$('.klink-linknew', panelDocument)
    this.$btnText = editor.$('.checkbox-text', panelDocument)
    this.$url = editor.$('.klink-url', panelDocument)
    this.$submit = editor.$('#klink-submit', panelDocument)
    this.$form = editor.$('.klink-form', panelDocument)
    this.$unlink = editor.$('#klink-unlink', panelDocument)
  }

  setEvent() {
    this.$btn.on('click', e => {
      e.stopPropagation()
    })
    this.$btnText.on('click', e => {
      e.stopPropagation()
    })
    this.$url.on('keyup', this.checkStatus)
    this.$submit.on('click', this.onSubmit)
    this.panel.on('keydown', this.onWindowKeydown)
    this.$form.on('reset', this.onCancel)
    this.$unlink.on('click', this.onUnlink)
    // window.addEventListener('scroll', e => {
    //   console.log(window.scrollY, this.panel._lastLayoutRect);
    //   // this.panel.moveBy(0, )
    //   console.log(this.panel)
    // })
  }

  removeEvent() {
    this.$btn.off('click')
    this.$btnText.off('click')
    this.$url.off('keyup')
    this.$submit.off('click')
    this.panel.off('keydown')
    this.$form.off('reset')
    this.$unlink.off('click')
    this.$unlink[0].disabled = true
  }

  setData() {
    const linkElm = LinkUtils.getAnchorElement(this.editor)
    const dom = this.editor.dom

    let url = ''
    let checked = true
    if (linkElm) {
      url = dom.getAttrib(linkElm, 'href')
      checked = dom.getAttrib(linkElm, 'target') === '_blank'
      this.$unlink[0].disabled = false
    }
    this.$url[0].value = url
    this.$btn[0].checked = checked
  }

  @autobind
  checkStatus() {
    const value = this.$url[0].value
    if (LinkUtils.isValidHref(value)) {
      this.$submit.removeAttr('disabled')
    } else {
      this.$submit.attr('disabled', true)
    }
  }

  @autobind
  onSubmit(e) {
    e.preventDefault()
    const value = this.$url[0].value
    const valid = LinkUtils.isValidHref(value)
    if (!valid) {
      return
    }

    const editor = this.editor
    const href = LinkUtils.makeValidHref(value)
    console.log(this.$btn[0], this.$btn[0].checked, typeof this.$btn[0].checked)
    const linkNew = this.$btn[0].checked

    const linkElm = LinkUtils.getAnchorElement(editor)

    let linkAttrs = {
      href: href,
      target: linkNew ? '_blank' : null,
    }

    if (linkElm) {
      editor.undoManager.transact(function() {
        editor.focus()
        editor.dom.setAttribs(linkElm, linkAttrs)
        editor.selection.select(linkElm)
        editor.undoManager.add()
      })
    } else {
      if (this.rng.collapsed) {
        editor.focus()
        editor.selection.setRng(this.rng)
        const targetAttr = linkNew ? ' target="_blank"' : ''
        editor.insertContent(`<a href="${href}"${targetAttr} rel="noopener" data-mce-href="${href}">${href}</a>`)
      } else {
        editor.focus()
        editor.selection.setRng(this.rng)
        editor.execCommand('mceInsertLink', false, linkAttrs)
      }
    }
    this.panel.hide()
  }

  @autobind
  onCancel(e) {
    this.panel.hide()
  }

  @autobind
  onUnlink(e) {
    const editor = this.editor
    const linkElm = LinkUtils.getAnchorElement(editor)

    if (linkElm) {
      editor.undoManager.transact(function() {
        editor.focus()
        editor.execCommand('Unlink')
      })
    } else {
      editor.focus()
      editor.selection.setRng(this.rng)
      editor.execCommand('Unlink', false)
    }
    this.panel.hide()
  }
}

export default LinkPanel
