import querystring from 'query-string'

const MAP_TYPE = {
  BASIC: 'basic',
  FULL: 'full'
}

const makeMode = mapType => {
  switch(mapType) {
    case MAP_TYPE.BASIC:
      return 'B'
    case MAP_TYPE.FULL:
    default:
      return 'F'
  }
}

const plugin = (editor) => {
  let targetUrl = editor.settings.maps.attach_map_url || ""
  let mapType = editor.settings.maps.map_type || 'data'
  let mapUrl = editor.settings.maps.map_url || 'http://dmaps.daum.net/map_include/map_editor/v3/iframe.html'

  editor.addButton('maps', {
    icon: 'maps',
    tooltip: '지도',
    cmd: 'openMap'
  })

  editor.addCommand('openMap', (data) => {
    window.open('https://attach.map.daum.net/?' + querystring.stringify({
        mode: makeMode(mapType),
        appendType: 'S',
        target: targetUrl
      }), "attach_map", "width=900px,height=600px")
  })

  editor.addCommand('attachMap', (data) => {
    editor.undoManager.ignore(() => {
      const id = `maps_${Date.now()}`
      editor.insertContent(editor.dom.createHTML('figure', {
        "id": id,
        'data-ke-type': 'map',
        'data-maps-data': querystring.stringify(data),
        'style': 'width:540px;height:350px;',
        'contentEditable': false
      }))
      editor.nodeChanged()
      appendMapImage(id, JSON.parse(data.mapInfo))
    })
  })

  const getStaticMapImageSrc = (mapInfo) => {
    const apiUrl = 'https://ssl.daumcdn.net/map3/staticmap/image?'
    let markersArray = [], markerQueryString = ''
    if (mapInfo.markInfo) {
      for (let i = 0 ; i < mapInfo.markInfo.length ; i++) {
        markersArray.push(querystring.stringify({
          markers: `symbol:sc_marker|location:${mapInfo.markInfo[i].x},${mapInfo.markInfo[i].y}`
        }))
      }
    }
    if (markersArray.length > 0) {
      markerQueryString = markersArray.join('&')
    }
    
    let mapQueryString = querystring.stringify({
      srs: 'WCONGNAMUL',
      size: 540 + 'x' + 350,
      lv: mapInfo.mapLevel,
      center: mapInfo.mapCenterX + ',' + mapInfo.mapCenterY,
    })

    return apiUrl + mapQueryString + '&' + markerQueryString
  }

  const appendMapImage = (id, mapInfo) => {
    const imageSrc = getStaticMapImageSrc(mapInfo)
    const imageElement = editor.dom.create('img', { src: imageSrc })
    editor.dom.$('#' + id).append(imageElement)
  }

  editor.on('SetContent', e => {
    const $ = editor.dom.$
    $('[data-maps-data]', e.node).each((idx, elm) => {
      if (elm.contentEditable != 'false') {
        const $elm = $(elm)
        let id = elm.id? elm.id : `maps_${Date.now()}_${idx}`
        const mapdata = $elm.attr('data-maps-data')

        const wrap = editor.dom.create('figure', {
          "id": id,
          "data-ke-type": 'map',
          "data-maps-data": mapdata,
          'contentEditable': false,
          'style': 'width:540px;height:350px;'
        })
        const mapDataObj = querystring.parse(mapdata)
        const mapInfo = JSON.parse(mapDataObj.mapInfo)
        const imageSrc = getStaticMapImageSrc(mapInfo)
        wrap.innerHTML = `<img src="${imageSrc}">`
        elm.parentNode.replaceChild(wrap, elm)
      }
    })
  })

  editor.on('PreProcess', e => {
    const $ = editor.dom.$
    $('[data-maps-data]', e.node).each((idx, elm) => {
      const $elm = $(elm)
      const id = $elm.attr('id')
      const data = $elm.attr('data-maps-data')
      const src = `${mapUrl}?id=${id}`

      const wrap = editor.dom.create('iframe', {
        "id": id,
        "data-ke-type": 'map',
        "mapdata": data,
        "data-maps-data": data,
        "width": "540px",
        "height": "350px",
        "frameborder": '0',
        "scrolling": 'no',
        "src": src
      })
      elm.outerHTML = wrap.outerHTML
    })
  })
}

export default plugin;
