import render from './render'

function insertEmptyParagraph(editor, rng, refernceNode) {
  const p = editor.dom.create('p')
  p.innerHTML = '<br data-mce-bogus="1">'
  editor.dom.insertAfter(p, refernceNode)

  rng.setStart(p, 0)
  rng.setEnd(p, 0)
  editor.selection.setRng(rng)
}

export function makeOpengraph(editor, targetNode, rng) {
  if (!targetNode) {
    return
  }

  const href = targetNode.getAttribute('href')
  const url = href.startsWith('http') ? href : 'http://' + href
  const curNode = editor.dom.getParent(editor.selection.getNode(), 'body > *')

  if (editor.dom.is(curNode, 'table')) {
    insertEmptyParagraph(editor, rng, targetNode)
    return
  }

  if (
    curNode.nextSibling &&
    curNode.nextSibling.nodeName === 'FIGURE' &&
    curNode.nextSibling.getAttribute('data-ke-type').startsWith('opengraph')
  ) {
    const dataNode = curNode.nextSibling.querySelector('a') || curNode.nextSibling
    if (dataNode && dataNode.getAttribute('data-original-url') === url) {
      insertEmptyParagraph(editor, rng, curNode)
      return
    }
  }

  makeOpengraphWithUrl(editor, url, targetNode, rng)
}

export function makeOpengraphWithUrl(editor, url, targetNode?, rng?) {
  if (!targetNode) {
    targetNode = editor.selection.getNode()
    rng = editor.selection.getSel().getRangeAt(0)
  }

  const figure = document.createElement('figure')
  figure.setAttribute('contenteditable', 'false')
  figure.className = 'og-loading'
  figure.textContent = '불러오는 중입니다...'
  editor.dom.insertAfter(figure, editor.dom.getParent(targetNode, 'body > *, body > div > *'))

  if (!figure.nextSibling) {
    insertEmptyParagraph(editor, rng, figure)
  }

  editor.settings.opengraph.fetch_handler(url, res => {
    res['original_url'] = url

    if (res.code !== 0) {
      editor.dom.remove(figure)
    } else {
      figure.innerHTML = render(res, figure)
      figure.removeAttribute('class')
      if (figure.getAttribute('data-video-host')) {
        const imageElem = figure.querySelector('img')
        imageElem.style.width = '100%'
        imageElem.onload = e => {
          imageElem.setAttribute('width', imageElem.offsetWidth.toString())
          imageElem.setAttribute('height', imageElem.offsetHeight.toString())
          imageElem.style.width = ''
          editor.nodeChanged();
        }
      }
    }
    editor.undoManager.add()
  })
}
