interface AttributesModifier {
  validElements: string[];
  extendedValidElements: {[key: string]: any};
  validAttributes: string[];
  validColors: string[];
  lookup: {[key: string]: any};
}

class AttributesModifier {
  constructor(settings) {
    this.validElements = settings.kPaste.valid_elements;
    this.extendedValidElements = settings.kPaste.extended_valid_elements;
    this.validAttributes = settings.kPaste.valid_attributes;
    this.validColors = settings.colorPalette.color_map_preset;

    this.lookup = {
      'TABLE': this.modifyTable,
      'SPAN': this.modifySpan,
      'FIGURE': this.modifyFigure,
    };
    this.makeLookup();
  }

  makeLookup() {
    for(const modifier in this.lookup) {
      this.lookup[modifier] = this.lookup[modifier].bind(this);
    }
  }

  modifyAttributes(node, useOriginal) {
    useOriginal = useOriginal || false;
    const clonedNode = useOriginal ? node : node.cloneNode(false);

    if(!clonedNode.attributes) {
      return clonedNode;
    }
    const validAttributes = this.validAttributes;
    let pos = 0;
    while(clonedNode.attributes[pos]) {
      const attr = clonedNode.attributes[pos];
      if(!this.validAttributes.includes(attr.name)) {
        clonedNode.removeAttribute(attr.name);
      }
      else {
        pos++;
      }
    }
    
    if(clonedNode.hasAttribute('data-mce-style')) {
      clonedNode.setAttribute('style', clonedNode.getAttribute('data-mce-style')); 
    }
  
    if(this.lookup[clonedNode.nodeName]) {
      this.lookup[clonedNode.nodeName](clonedNode, node);
    }
    return clonedNode;
  }
  
  modifyTable(node) {
    node.style['border-collapse'] = "collapse";
    node.style.width = "100%";
    node.setAttribute('border', 1);
  }

  modifySpan(clonedNode, originalNode) {
    const validColors = this.validColors;
    const nodeColor = originalNode.style.color; 
    if(!nodeColor) {
      return clonedNode;
    }
    this.validColors.forEach(color => {
      const colorString = `rgb(${parseInt(color.slice(1,3), 16)}, ${parseInt(color.slice(3,5), 16)}, ${parseInt(color.slice(5,7), 16)})`
      if(nodeColor === colorString) {
        clonedNode.style.color = nodeColor;
      }
    })
  }

  modifyFigure(node) {
    if(node.id) {
      node.id = node.id.replace(/_.*_?/i, `_${Date.now()}`);
    }
  }
}

export default AttributesModifier;