class Validator {
  invalidTableChildrenList: string[];
  validElements: string[];
  extendedValidElements: {[key: string]: any};
  validAttributes: string[];
  validColors: string[];
  modifier: any;

  constructor(settings, modifier) {
    this.invalidTableChildrenList = settings.kPaste.invalid_table_children;
    this.validElements = settings.kPaste.valid_elements;
    this.extendedValidElements = settings.kPaste.extended_valid_elements;
    this.validAttributes = settings.kPaste.valid_attributes;
    this.validColors = settings.colorPalette.color_map_preset;

    this.modifier = modifier;
  }

  validateType(node) {
    for(let type of this.validElements) {
      if(node.nodeName === type) {
        return this.modifier.modifyAttributes(node);
      }
    }
    if(this.extendedValidElements[node.nodeName]) {
      const c = node.cloneNode(false);
      const m = document.createElement(this.extendedValidElements[node.nodeName]);
      m.innerHTML = c.innerHTML;
      return m;
    }
  
    if(node.nodeName === 'DIV') {
      if(node.getAttribute('data-ke-type') === 'moreLess') {
        return node;
      }
      else {
        return document.createElement('p');
      }
    }
    return null;
  }

  isInvalidTableChildren(node) {

    for(let type of this.invalidTableChildrenList) {
      if(node.nodeName === type) {
        return true;
      }
    }

    if(node.nodeName === 'DIV' && node.getAttribute('data-ke-type') === 'moreLess') {
      return true;
    }
    return false;
  }

  static isValidSpan(node) {
    while(node) {
      if(node.childNodes.length === 1 && ['SPAN', '#text', 'B', 'U', 'S', 'I'].includes(node.firstChild.nodeName)) {
        node = node.firstChild;
      }
      else if(node.nodeName === '#text') {
        return true;
      }
      else {
        return false;
      }
    }
  }
}

export default Validator;