const checkCondition = targetNode => (targetNode.nodeName === 'FIGCAPTION' || targetNode.nodeName === 'BLOCKQUOTE')

const getContainer = commonContainer => {
  if(commonContainer.nodeName === '#text') {
    return commonContainer;
  }
  else {
    if(commonContainer.firstChild) {
      return commonContainer.firstChild;
    }
    else {
      return commonContainer.appendChild(document.createTextNode(''));
    }
  }
}

// 특정 요소에는 텍스트만 붙여넣어지게끔 처리해야 하는데, 이 때 클립보드 내용을 텍스트만 발라내는 동작.
// 트리 전체를 탐색하지 않고 1단계만 훑어서 텍스트를 뽑아낸 뒤 스페이스, 혹은 <br>로 묶는다.
const skimAndInsert = spaceDelim => (editor, tree) => {
  const parent = document.createElement('div');
  const len = tree.childNodes.length;
  if(spaceDelim) {
    let str = '';
    for(let i = 0; i < len; i++) {
      if(!tree.childNodes[i].textContent) continue;
      str += tree.childNodes[i].textContent;
      if(i !== len-1) {
        str += ' ';
      }
    }
    const rng = editor.selection.getSel().getRangeAt(0);
    const node = editor.selection.getNode();

    // fierfox에서 figcaption 전체를 선택하고 붙여넣을 경우 figcaption 자체가 삭제되는 현상 처리
    if(node.nodeName === 'FIGURE') {
      const figcaption = document.createElement('figcaption');
      figcaption.appendChild(document.createTextNode(str));
      node.appendChild(figcaption);
      editor.selection.setCursorLocation(figcaption, 1);
      editor.undoManager.add();
      return;
    }
    else if (node.nodeName === 'FIGCAPTION') {
      if(node.firstChild && node.firstChild.nodeName === 'BR') {
        editor.dom.remove(node.firstChild);
      }
    }

    const container = getContainer(rng.commonAncestorContainer);
    let textContent = container.textContent;
    const { sOffset, eOffset } = rng.startOffset < rng.endOffset ? { sOffset: rng.startOffset, eOffset: rng.endOffset } : { sOffset: rng.endOffset, eOffset: rng.startOffset };
    
    textContent = textContent.slice(0, sOffset) + str + textContent.slice(eOffset, textContent.length);
    container.textContent = textContent;

    editor.selection.setCursorLocation(container, sOffset + str.length);
    editor.undoManager.add();
  }
  else {
    for(let i = 0; i < len; i++) {
      parent.appendChild(document.createTextNode(tree.childNodes[i].textContent));
      parent.appendChild(document.createElement('br'));
    }
    editor.insertContent(parent.innerHTML, {paste: "kPaste"});
    editor.dom.remove(parent);
    editor.undoManager.add();
  }
}

const getImmediateHandler = targetNode => {

  if(!checkCondition(targetNode)) return;

  const targets = {
    'FIGCAPTION': skimAndInsert(true),
    'BLOCKQUOTE': skimAndInsert(false),
  };

  return targets[targetNode.nodeName]; 
}

export default getImmediateHandler;