import htmlParser from './htmlParser'
import { msWordContent, isMSWord } from './regex'
import * as bowser from 'bowser'
import getImmediateHandler from './getImmediateHandler'

const getAsPromise = str =>
  new Promise(resolve => {
    str.getAsString(item => {
      resolve(item)
    })
  })

const extractHtml = async item => {
  let returnee
  if (item.getAsString) {
    returnee = await getAsPromise(item)
  } else {
    returnee = item
  }
  return returnee
}

const extractMS = item => {
  const match = msWordContent(item)
  if (match) {
    item = match[match.length - 2]
  }
  if (isMSWord(item)) {
    item = item.replace(/(\r)/gi, '')
    item = item.replace(/<(\/)?(v|o):(.|\r|\n)*?(\/)?>/gi, '')
    item = item.replace(/<!(--)?\[(end)?if(.|\n|\r)*?>/gi, '')
  }
  return item
}

const handleHtml = (editor, item, targetNode, deleteRange, validator, modifier) => {
  // 만약 붙여넣을 html에 이미지가 있을 경우, 개별 이미지 업로드 완료시마다 실행될 콜백.
  // 마지막 이미지까지 업로드 될 경우 내용을 삽입한다.
  const makeImageUploadCallback = resultList => {
    let len = imgList.length
    return result => {
      resultList.push({
        id: result.options.targetId,
        url: result.data ? result.data.url : undefined,
      })

      len--
      const bar = <HTMLElement>document.getElementsByClassName('mce-white-progress-percentage')[0]
      const percentText = document.getElementsByClassName('mce-white-percentage-number')[0]
      const percent = ((imgList.length - len) / imgList.length) * 100

      bar.style.width = percent + '%'
      percentText.textContent = Math.round(percent) + '%'

      if (len === 0) {
        finishHandling(resultList)
        editor.windowManager.close()
      }
    }
  }

  // 붙여넣을 html에 대한 처리가 모두 끝난 뒤 실제로 내용을 삽입하는 함수.
  const finishHandling = (uploadedImageList?) => {
    if (!cleanedDiv.firstChild) {
      return
    }

    if (uploadedImageList) {
      uploadedImageList.forEach(img => {
        const imgNode = cleanedDiv.querySelector('#' + img.id)
        if (img.url) {
          imgNode.src = img.url
          imgNode.removeAttribute('id')
        } else {
          editor.dom.remove(imgNode.parentNode)
        }
      })
    }

    const curRange = editor.selection.getRng().cloneRange()
    del(curRange)

    editor.undoManager.transact(() => {
      const sel = editor.selection.getSel()
      const focusNode = sel.focusNode
      const focusRootParent = editor.dom.getParent(focusNode, 'body > *')
      const children = cleanedDiv.childNodes

      // 만약 붙여넣어질 html의 첫 요소 혹은 마지막 요소가 현재 커서가 위치한 요소와 같은 경우, 분리하지 말고 같이 묶어주기 위한 로직
      const isSplit = focusNode.nodeName === '#text' && sel.focusOffset !== focusNode.length && sel.focusOffset !== 0
      const isMatchedAtStart =
        children[0] && focusRootParent ? children[0].nodeName === focusRootParent.nodeName : false
      const isMatchedAtEnd =
        children[children.length - 1] && focusRootParent
          ? children[children.length - 1].nodeName === focusRootParent.nodeName
          : false

      if (isMatchedAtStart) {
        const firstChild = children[0]
        editor.insertContent(firstChild.innerHTML, { paste: 'kPaste' })
        editor.dom.remove(firstChild)
      }

      editor.insertContent(cleanedDiv.innerHTML, { paste: 'kPaste' })
      const curNode =
        sel.focusNode.nodeName === '#text' ? editor.dom.getParent(sel.focusNode, 'body > *') : sel.focusNode

      // 이것도 좀 보긴 해야할듯. 돌아가긴 하는데 뭔가 이상하다.
      if (isSplit && isMatchedAtEnd && curNode.nextSibling) {
        if (['P', 'H1', 'H2', 'H3'].includes(curNode.nodeName)) {
          while (curNode.nextSibling.nodeName === curNode.nodeName && curNode.nextSibling.childNodes.length) {
            curNode.appendChild(curNode.nextSibling.firstChild)
          }
        }
      }
    })
    editor.dom.remove(originalDiv)
    editor.dom.remove(cleanedDiv)

    // contenteditable = false 앞에 사진을 붙여넣을 경우 이미지 로딩으로 인해 caret 위치가 틀어지는데, 이를 막기 위한 후처리
    const afterSelection = editor.selection.getNode()
    if (afterSelection.getAttribute('data-mce-caret') === 'before') {
      editor.selection.select(afterSelection.nextSibling || afterSelection.previousSibling)
    }
  }

  const bulkUploadImage = () => {
    const len = imgList.length
    const resultList = []
    const ua = bowser.name

    // safari와 ie의 경우 file이 아닌 blob 포맷으로 업로드를 해야한다.
    const shouldUploadAsBlob = ['Safari', 'Internet Explorer'].includes(ua)
    const imageUploadCallback = makeImageUploadCallback(resultList)
    imgList.forEach(img => {
      let uploadUrl = img.src
      const targetId = img.id
      if (!uploadUrl.startsWith('data:image')) {
        const match = /.*daumcdn.net\/thumb.*?\?fname=(.*)/i.exec(uploadUrl)
        if (match) {
          uploadUrl = decodeURIComponent(match[match.length - 1])
        }
      }
      editor.undoManager.ignore(() => {
        editor.execCommand('KImageUploadUrl', {
          url: uploadUrl,
          targetId,
          isInvisible: true,
          callback: imageUploadCallback,
          shouldUploadAsBlob,
        })
      })
    })

    editor.windowManager.open({
      width: 800,
      height: 100,
      classes: 'white-dialog',
      items: [
        {
          type: 'container',
          html: `<div class='mce-white-body'><p class='mce-white-message'>붙여넣기 및 이미지 업로드 중입니다</p><div class='mce-white-progress-border'><div class='mce-white-progress-percentage'></div></div><p class='mce-white-percentage-number'>0%</p></div>`,
        },
      ],
      buttons: [],
    })
  }

  const del = (rng?) => {
    if (deleteRange) {
      deleteRange()
    }
    // else {
    //   editor.selection.setRng(rng);
    // }
  }

  const deleteStyleNode = pasteString => {
    return pasteString.replace(/(<style([^<]+)<\/style>)/, '')
  }

  // style tag 제거
  item = deleteStyleNode(item)

  item = extractMS(item)
  // 파싱을 위해 원본 html을 dom tree로 만든다.
  const originalDiv = document.createElement('div')
  originalDiv.innerHTML = item

  // 파싱하지 않고 바로 처리해도 되는 조건에 부합한다면 이 단계에서 바로 처리하고 파싱을 수행하지 않는다.
  const handleImmediately = getImmediateHandler(editor.selection.getNode())
  if (handleImmediately) {
    del()
    handleImmediately(editor, originalDiv)
    editor.dom.remove(originalDiv)
    return
  }

  const { cleanedDiv, imgList } = htmlParser(editor, originalDiv, validator, modifier, targetNode)
  if (targetNode && editor.dom.is(targetNode, 'figure')) {
    targetNode.removeAttribute('data-mce-selected')
  }

  // 붙여넣어질 html을 파싱한 결과 이미지가 있을 경우, 삽입 전에 이미지 업로드를 수행한다.
  if (imgList.length) {
    bulkUploadImage()
  } else {
    finishHandling()
  }
}

export { handleHtml, extractHtml }
