import recipeLookup from './recipe';

const htmlParser = (editor, tree, validator, modifier, targetNode) => {

  const counter = {imgCnt: 0};
  const imgList = [];
  const validElements = editor.settings.kPaste.valid_elements;
  const extendedValidElements = editor.settings.kPaste.extended_valid_elements;
  const validAttributes = editor.settings.kPaste.valid_attributes;
  const validColors = editor.settings.colorPalette.color_map_preset;

  const recursiveTraversal = (node, parent) => {
    
    if(!node) {
      return parent;
    }

    if(validator.isInvalidTableChildren(node)) {
      if(editor.dom.is(targetNode, 'table')) {
        return parent;
      }
      if(editor.dom.getParent(node, 'td')) {
        return parent;
      }
    }

    if(recipeLookup[node.nodeName]) {
      const recipe = recipeLookup[node.nodeName];
      const result = recipe(node, parent, {counter, imgList, modifier, targetNode});
      if(result) {
        return;
      }
    }

    const curRoot = validator.validateType(node, {validElements, extendedValidElements}) || parent;

    if(node.firstChild) {
      if(curRoot !== parent) {
        parent.appendChild(curRoot);
      }

      const childArray = Array.from(node.childNodes);
      childArray.forEach(child => {
        recursiveTraversal(child, curRoot)
      });
    }
  
    if(curRoot.nodeName === 'P' && !curRoot.firstChild) {
      editor.dom.remove(node);
    }
    return parent;
  }

  const newRoot = document.createElement('div');
  // newRoot.className = 'pasteroot';
  const result = recursiveTraversal(tree, newRoot);
  
  return {cleanedDiv: result.firstChild, imgList};
}

export default htmlParser;
