import { regexString, isUrlInText } from './regex';

const scriptRecipe = (node, parent) => {
  return true;
};

const tableRecipe = (node, parent, { modifier, targetNode }) => {
  if(targetNode && targetNode.nodeName === 'TABLE') {
    return true;
  }
  node = modifier.modifyAttributes(node, true);
  return false;
}

const brRecipe = (node, parent) => {

  if(node.hasAttribute('data-mce-bogus') || node.classList.length) {
    return true;
  }
  parent.appendChild(document.createElement('br'));
  return true;
};

const imgRecipe = (node, parent, {counter, imgList, modifier}) => {
  let curParent = node.parentNode;
  while(curParent) {
    if(curParent.nodeName === 'FIGURE' && curParent.hasAttribute('data-ke-type')) {
      break;
    }
    curParent = curParent.parentNode;
  }
  if(curParent) {
    parent.appendChild(curParent);
    return true;
  }
  else {
    let pos = 0;
    const clonedNode = modifier.modifyAttributes(node);
    clonedNode.src = node.src;
    
    const figureNode = document.createElement('figure');
    figureNode.setAttribute('data-ke-type', 'image');
    figureNode.appendChild(clonedNode);
    if (window.tinymce.settings.kImage.is_caption) {
      figureNode.appendChild(document.createElement('figcaption'));
    }
    parent.appendChild(figureNode);
    clonedNode.id = `img_${Date.now()}_${counter.imgCnt++}`;
    imgList.push(clonedNode);
  }
};

const textRecipe = (node, parent) => {
  if(isUrlInText(node.textContent)) { // 현재 노드의 텍스트 안에 url이 포함될 경우 처리
    let innerHTMLString = node.textContent;
    innerHTMLString = innerHTMLString.replace(regexString.isUrlInText, (match, ...args) => {
      return `<a href=${args[0]}>${args[0]}</a>`
    })
    parent.innerHTML += innerHTMLString;
  }
  else {
    const textNode = document.createTextNode(node.textContent);
    parent.appendChild(textNode);
  }
  return true;
};

const figureRecipe = (node, parent, {modifier}) => {
  if(node.getAttribute('data-ke-type')) {
    modifier.modifyAttributes(node, true);
    
    Array.from(node.childNodes).forEach(function (child: HTMLElement) {
      
      if(child.nodeName === 'BR') {
        child.remove();
        return;
      }
      modifier.modifyAttributes(child, true);
    })
    parent.appendChild(node);
    return true;
  }
  return false;
}

const preRecipe = (node, parent) => {
  if(node.getAttribute('data-ke-type')) {
    node.id = `code_${Date.now()}`;
    parent.appendChild(node);

    return true;
  }
  return false;
}

const hrRecipe = (node, parent) => {
  parent.appendChild(node);
  return true;
}

const recipeLookup = {
  'SCRIPT': scriptRecipe,
  'FIGURE': figureRecipe,
  'BR': brRecipe,
  'IMG': imgRecipe,
  '#text': textRecipe,
  'PRE': preRecipe,
  'HR': hrRecipe,
  'TABLE': tableRecipe,
};


export default recipeLookup;