import * as Sentry from '@sentry/browser'
import Pkg from 'package.json'

const plugin = editor => {
  const isProduct: boolean = Pkg.name == 'keditor'
  const sentryDsn = {
    product: 'https://4db076eedefe4f49a336436600eb48ad@sentry.kakao.com/1390',
    demo: 'https://e82a0b9bd96f420e880cabca566d5461@sentry.kakao.com/1340',
  }
  const whitelistUrls = {
    product: [/https?:.+\/keditor\/.+\.js/, /https?:.+\/photos\.standalone\.min\.js/],
    demo: [/.+\.[jt]s/],
  }

  const isWhiteUrls = sentryEvent => {
    if (sentryEvent.exception.values && sentryEvent.exception.values.length > 0) {
      const exceptionValue = sentryEvent.exception.values[0]
      if (
        exceptionValue.stacktrace &&
        exceptionValue.stacktrace.frames &&
        exceptionValue.stacktrace.frames.length > 0
      ) {
        const lastExceptionFrame = exceptionValue.stacktrace.frames[exceptionValue.stacktrace.frames.length - 1]
        const allowWhiteUrls: Array<RegExp> = isProduct ? whitelistUrls.product : whitelistUrls.demo
        for (let i = allowWhiteUrls.length; i--; ) {
          if (allowWhiteUrls[i].test(lastExceptionFrame.filename)) {
            return true
          }
        }
        return false
      }
    }
  }

  const options = {
    dsn: isProduct ? sentryDsn.product : sentryDsn.demo,
    release: Pkg.version,
    autoBreadcrumbs: true,
    environment: isProduct ? 'product' : 'development',
    attachStacktrace: true,
    dist: editor.settings.theme,
    // whitelistUrls: isProduct ? whitelistUrls.product : whitelistUrls.demo,
    beforeSend: function(event) {
      if (isWhiteUrls(event)) {
        event.request = {
          headers: {
            'User-Agent': navigator.userAgent,
          },
          url: location.href,
        }
        return event
      }
    },
  }

  Sentry.init(options)
  window.addEventListener('error', function(e) {
    new Sentry.BrowserClient(options).captureException(e)
  })
}

export default plugin
