const plugin = editor => {
  const tablePlugin = editor.plugins.kTable

  if (!tablePlugin) {
    return false
  }

  const isGridOption = editor.settings.simpleTable && editor.settings.simpleTable.is_grid
  const align = (editor.settings.simpleTable && editor.settings.simpleTable.align) || 'left'

  const generateTableGrid = () => {
    let html = ''
    html = '<table role="grid" class="mce-grid mce-grid-border" aria-readonly="true">'
    for (let y = 0; y < 10; y++) {
      html += '<tr>'
      for (let x = 0; x < 10; x++) {
        html +=
          '<td role="gridcell" tabindex="-1"><a id="mcegrid' +
          (y * 10 + x) +
          '" href="#" ' +
          'data-mce-x="' +
          x +
          '" data-mce-y="' +
          y +
          '"></a></td>'
      }
      html += '</tr>'
    }
    html += '</table>'
    html += '<div class="mce-text-center" role="presentation">1 x 1</div>'
    return html
  }

  const selectGrid = (tx, ty, control) => {
    const table = control.getEl().getElementsByTagName('table')[0]
    let x, y, focusCell, cell, active
    const rtl = control.isRtl() || control.parent().rel === 'tl-tr'
    table.nextSibling.innerHTML = tx + 1 + ' x ' + (ty + 1)
    if (rtl) {
      tx = 9 - tx
    }
    for (y = 0; y < 10; y++) {
      for (x = 0; x < 10; x++) {
        cell = table.rows[y].childNodes[x].firstChild
        active = (rtl ? x >= tx : x <= tx) && y <= ty
        editor.dom.toggleClass(cell, 'mce-active', active)
        if (active) {
          focusCell = cell
        }
      }
    }
    return focusCell.parentNode
  }

  if (isGridOption) {
    editor.addButton('simpleTable', {
      icon: 'table',
      type: 'simplemenubutton',
      tooltip: 'Table',
      onshow() {
        selectGrid(0, 0, this.menu.items()[0])
      },
      onhide() {
        const elements = this.menu
          .items()[0]
          .getEl()
          .getElementsByTagName('a')
        editor.dom.removeClass(elements, 'mce-active')
        editor.dom.addClass(elements[0], 'mce-active')
      },
      menu: {
        items: [
          {
            type: 'container',
            html: generateTableGrid(),
            onPostRender(e) {
              this.lastX = this.lastY = 0
            },
            onmousemove(e) {
              const target = e.target
              let x, y
              if (target.tagName.toUpperCase() === 'A') {
                x = parseInt(target.getAttribute('data-mce-x'), 10)
                y = parseInt(target.getAttribute('data-mce-y'), 10)
                if (this.isRtl() || this.parent().rel === 'tl-tr') {
                  x = 9 - x
                }
                if (x !== this.lastX || y !== this.lastY) {
                  selectGrid(x, y, e.control)
                  this.lastX = x
                  this.lastY = y
                }
              }
            },
            onclick(e) {
              const self = this
              if (e.target.tagName.toUpperCase() === 'A') {
                e.preventDefault()
                e.stopPropagation()
                self.parent().cancel()

                editor.undoManager.transact(function() {
                  tablePlugin.insertTable(self.lastX + 1, self.lastY + 1)
                })
                editor.addVisual()
              }
            },
          },
        ],
        onPostRender(e) {
          e.control.fixed(true)
        },
      },
      onShowMenu() {
        if (align === 'center') {
          this.menu.moveRel(this.getEl(), ['bc-tc', 'tc-bc']);
        }
      }
    })
  } else {
    editor.addButton('simpleTable', {
      icon: 'table',
      tooltip: 'Table',
      onclick: () => {
        tablePlugin.insertTable(2, 2)
      },
    })
  }
}

export default plugin
