import autobind from 'autobind-decorator'

class VideoUploadDialog {
  constructor(editor) {
    this.editor = editor
    this.serviceName = editor.settings.video.service_name
    this.serviceType = editor.settings.video.service_type || 'kakao'
    this.openStatus = false
    this.win = null

    if (this.serviceType == 'daum') {
      this.uploadDomain = 'kakaotv.daum.net'
    } else {
      this.uploadDomain = 'tv.kakao.com'
    }

    window.addEventListener('message', this.handleReceiveMessage, false)
    window.addEventListener('keydown', this.handleWindowKeydown, false)
  }
  open() {
    const path = this.serviceType == 'daum' ? 'daum' : 'external'
    const height = this.serviceType == 'daum' ? 624 : 680
    this.win = this.editor.windowManager.open({
      title: 'Upload Kakao TV',
      items: [
        {
          type: 'container',
          html: `<div class="keditor-dialog" style="width:600px; height:${height}px">
					<iframe src="https://${this.uploadDomain}/uploader/${path}?service=${this.serviceName}" border="0" width="100%" />
        </div>`,
        },
      ],
      buttons: [],
      onClose: this.handleClose,
    })
    this.win.statusbar.remove()
  }

  close() {
    if (this.win) {
      this.win.close()
    }
  }

  @autobind
  handleClose() {
    this.win = null
  }

  @autobind
  handleReceiveMessage(e) {
    if (!this.win) {
      return
    }

    if (e.origin != `https://${this.uploadDomain}`) {
      return
    }

    const data = JSON.parse(e.data)
    if (data.eventType && data.eventType === 'attach') {
      const service = data.clip.reservedVideo.service
      const vid = data.clip.reservedVideo.vid
      const uploader_host = data.clip.reservedVideo.uploader
      const thumbnail = data.image.feed.replace('http://', 'https://')
      const url = `https://tv.kakao.com/embed/player/cliplink/${vid}@my?service=${service}`

      this.editor.undoManager.ignore(() => {
        this.editor.insertContent(
          `<figure data-ke-type="video" contenteditable="false" data-video-thumbnail="${thumbnail}" data-video-vid="${vid}" data-video-url="${url}" data-video-uploader-host="${uploader_host}" style="background:url(${thumbnail}) no-repeat"></figure>`
        )
      })
    }

    this.close()
  }

  @autobind
  handleWindowKeydown(e) {
    let keyCode = e.keyCode
    if (keyCode === 27) {
      this.close()
    }
  }
}

export default VideoUploadDialog
