import axios from 'axios'

class VideoUploader {
  constructor(editor) {
    this.editor = editor
    this.$progressbarNode = null
    this.$fragmentElement = null
    this.uploadUrl = editor.settings.video && editor.settings.video.upload_url
    this.uploadHandler = editor.settings.video && editor.settings.video.upload_handler

    if (!this.uploadUrl && !this.uploadHandler) {
      throw new Error('video.upload_url or video.upload_handler is required')
    }

    this.upload = this.upload.bind(this)
  }

  upload(file) {
    const id = 'video_' + new Date().getTime()
    this.editor.undoManager.ignore(() => {
      this.editor.insertContent(
        `<figure id="${id}" data-ke-type="video" class="uploading" contenteditable="false"><span class="text_upload">업로드 중<br>${
          file.name
        }</span></figure>`
      )
    })
    const $wrap = this.editor.dom.$(`#${id}`)

    const progressHandler = (loaded, total) => {
      const percent = Math.round((loaded / total) * 100)
      const _this = this
      this.$progressbarNode.css('width', percent + '%')
      if (percent == 100) {
        _this.editor.undoManager.ignore(() => {
          tinymce.util.Delay.setTimeout(() => {
            $wrap.html(
              '<span class="play_video"></span><span class="text_thumbnail">동영상 분석 및 썸네일 생성중</span>'
            )
          }, 500)
        })
      }
    }

    this.addProgressBar(id)

    const _this = this
    const callback = result => {
      $wrap.attr({
        'data-video-vid': result.vid,
        'data-video-thumbnail': result.thumbnail,
        'data-video-url': result.url,
        'data-video-uploader-host': result.uploader_host,
      })
      $wrap.html(`<div class="video_wrap"><img src="${result.thumbnail}" /><span class="play_video"></span></div>`)
      $wrap.removeClass('uploading')
      _this.editor.undoManager.add()
    }

    if (this.uploadHandler && typeof this.uploadHandler === 'function') {
      this.uploadHandler(file, callback, progressHandler)
    } else {
      this.uploadVideo(file, callback, progressHandler)
    }
  }

  addProgressBar(id) {
    const $videoWrap = this.editor.dom.$('#' + id)
    if (!this.$progressbarNode) {
      this.$progressbarNode = this.editor.dom.$('<span class="bar_progress"></span>')
    }
    $videoWrap.append(this.$progressbarNode)
  }

  uploadVideo(file, callback, progressHandler) {
    const config = {
      onUploadProgress: e => {
        if (progressHandler && typeof progressHandler === 'function') {
          progressHandler(e.loaded, e.total)
        }
      },
    }

    const formdata = new FormData()
    formdata.append('video', file, file.name)

    axios
      .post(this.uploadUrl, formdata, config)
      .then(res => res.data)
      .then(callback)
      .catch(res => {
        console.error('video upload error', res)
      })
  }
}

export default VideoUploader
