/**
 * Theme.js
 *
 * Released under LGPL License.
 * Copyright (c) 1999-2016 Ephox Corp. All rights reserved
 *
 * License: http://www.tinymce.com/license
 * Contributing: http://www.tinymce.com/contributing
 */

// import ThemeManager from 'tinymce/core/api/ThemeManager';
import ThemeApi from './api/ThemeApi'
import Api from 'tinymce/ui/Api'
import FormatControls from 'tinymce/ui/FormatControls'
import NodeChangedSelector from './components/NodeChangedSelector'
import InsertEmptyParagrash from './components/InsertEmptyParagraph'
import KeyEvent from './components/KeyEvent'
import BeforeExecCommand from './components/BeforeExecCommand'
import TiaraClick from './components/TiaraClick'
import SimpleMenuButton from './ui/SimpleMenuButton'
import Factory from 'tinymce/core/api/ui/Factory'

declare let window: any

const tinymce = window.tinymce || {}
const ThemeManager = tinymce.util.Tools.resolve('tinymce.ThemeManager')

Api.registerToFactory()
Api.appendTo(tinymce)

Factory.add('simplemenubutton', SimpleMenuButton)
tinymce.ui.simplemenubutton = SimpleMenuButton

ThemeManager.add('classic', function(editor) {
  FormatControls.setup(editor)
  NodeChangedSelector.register(editor)
  InsertEmptyParagrash.register(editor)
  KeyEvent.register(editor)
  BeforeExecCommand.register(editor)
  TiaraClick.register(editor)
  return ThemeApi.get(editor)
})

export default function() {}
