const register = editor => {
  editor.on('click', e => {
    const target = e.target
    if (['BODY', 'DIV'].indexOf(target.nodeName) > -1) {
      insertEmptyParagraphProcess(e)
    }
  })

  const getBottomElementByClickOffsetY = (e, clickOffsetY) => {
    const $figureElem = editor.dom.$('figure, hr, blockquote, div, table, pre', e.target)
    for (let i = 0, l = $figureElem.length; i < l; i++) {
      if ($figureElem[i].offsetTop > clickOffsetY) {
        return $figureElem[i]
      }
    }
  }

  const insertEmptyParagraphProcess = e => {
    let clickOffsetY = e.offsetY
    if (e.target.nodeName != 'BODY') {
      clickOffsetY = e.target.offsetTop + e.offsetY
    }
    const $clickBottomFigrueElem = editor.$(getBottomElementByClickOffsetY(e, clickOffsetY))
    if ($clickBottomFigrueElem[0]) {
      const $previousElemFromFirgureElem = $clickBottomFigrueElem.prev()
      if (
        $previousElemFromFirgureElem[0] &&
        isMatchNodeName($previousElemFromFirgureElem[0], ['FIGURE', 'HR', 'BLOCKQUOTE', 'DIV', 'TABLE', 'PRE']) &&
        $previousElemFromFirgureElem[0].offsetTop + $previousElemFromFirgureElem[0].offsetHeight < clickOffsetY
      ) {
        insertEmptyParagraph($previousElemFromFirgureElem, 'after')
      } else if (!$previousElemFromFirgureElem[0] || $previousElemFromFirgureElem.attr('data-mce-caret')) {
        insertEmptyParagraph($clickBottomFigrueElem, 'before')
      }
    } else if (isMatchNodeName(e.target.lastChild, ['FIGURE', 'HR', 'BLOCKQUOTE', 'DIV', 'TABLE', 'PRE'])) {
      if (
        (!e.target.lastChild.getAttribute('data-mce-bogus') ||
          e.target.lastChild.className.split('mce-visual-caret').length > 1) &&
        e.target.lastChild.offsetTop + e.target.lastChild.offsetHeight < clickOffsetY
      ) {
        insertEmptyParagraph(e.target.lastChild, 'after')
      }
    }
  }

  const isMatchNodeName = (node: Node, nodeNameArray: Array<String>) => {
    if (!node) {
      return false
    }
    return nodeNameArray.indexOf(node.nodeName) > -1
  }

  const insertEmptyParagraph = (elem, beforeAfter) => {
    const $elem = editor.$(elem)
    const newParagraph = editor.$('<p><br data-mce-bogus="1"></p>')[0]
    if (beforeAfter == 'before') {
      $elem.before(newParagraph)
    } else if (beforeAfter == 'after') {
      $elem.after(newParagraph)
    }
    editor.selection.setCursorLocation(newParagraph)
    setCursorLocationFirst(newParagraph)
    editor.undoManager.add()
  }

  const setCursorLocationFirst = targetElement => {
    const range = editor.selection.getRng()
    range.setStart(targetElement, 0)
    range.setEnd(targetElement, 0)
    editor.selection.getSel().removeAllRanges()
    editor.selection.getSel().addRange(range)
  }
}

export default {
  register,
}
