const register = editor => {
  const textItemSelector = 'p, h1, h2, h3, h4, h5, h6'
  editor.on('keydown', e => {
    keyDownHandler(e)
  })

  const keyDownHandler = e => {
    if (e.keyCode == 40) {
      arrowDownController(e)
    } else if (e.keyCode == 38) {
      arrowUpController(e)
    } else if (e.keyCode == 8) {
      backspaceController(e)
    } else if (e.keyCode == 46) {
      deleteController(e)
    } else if (e.keyCode == 13) {
      enterController(e)
    }
  }

  const arrowDownController = e => {
    const selectionNode = editor.selection.getNode()
    if (selectionNode.nodeName == 'FIGURE' && selectionNode.nextSibling && selectionNode.nextSibling.nodeName == 'FIGURE') {
      e.preventDefault()
      setCursorLocation(selectionNode.nextSibling)
      scrollToSelection(selectionNode.nextSibling, e.key)
    } else if (selectionNode.nodeName == 'FIGURE' && selectionNode.nextSibling) {
      e.preventDefault()
      setCursorLocation(selectionNode.nextSibling)
      scrollToSelection(selectionNode.nextSibling, e.key)
    }
  }

  const arrowUpController = e => {
    const selectionNode = editor.selection.getNode()
    const focusNode = editor.selection.getSel().focusNode
    if (selectionNode && selectionNode.previousSibling && focusNode && focusNode.nodeName == '#text'
      && (!focusNode.previousElementSibling || focusNode.previousElementSibling.nodeName != 'BR')
      && selectionNode.previousSibling.nodeName == 'FIGURE') {
      e.preventDefault()
      setCursorLocation(selectionNode.previousSibling)
      scrollToSelection(selectionNode.previousSibling, e.key)
    } else if (selectionNode.nodeName == 'FIGURE' && selectionNode.previousSibling && selectionNode.previousSibling.nodeName == "FIGURE") {
      e.preventDefault()
      setCursorLocation(selectionNode.previousSibling)
      scrollToSelection(selectionNode.previousSibling, e.key)
    }
  }

  const backspaceController = e => {
    const $ = editor.dom.$
    const selectionNode = editor.selection.getNode()
    if (selectionNode.tagName == "FIGURE") {
      e.preventDefault()
      removeFigureAfterSelection(selectionNode)
    } else if (selectionNode.nodeName == "FIGCAPTION") {
      // 이미지 캡션의 keyEvent 옮겨오기
    } else if (isFirstOfParagraphCaret()) {
      const $paragraphElement = $(selectionNode).closest(textItemSelector)
      const previousElement = $paragraphElement.prev()[0]
      if (previousElement && previousElement.nodeName == 'FIGURE') {
        e.preventDefault()
        setCursorLocation(previousElement)
        if ($paragraphElement.text() == '\n') {
          $paragraphElement.remove()
        }
      } else if (!previousElement && $paragraphElement.text() == '\n' && $paragraphElement.next()[0]) {
        e.preventDefault()
        setCursorLocation($paragraphElement.next()[0])
        $paragraphElement.remove()
      }
    }
  }

  const deleteController = e => {
    const $ = editor.dom.$
    const selectionNode = editor.selection.getNode()
    if (selectionNode.nodeName == "FIGURE") {
      e.preventDefault()
      removeFigureAfterSelection(selectionNode)
    } else if (selectionNode.nodeName == "FIGCAPTION") {
      // 이미지 캡션의 keyEvent 옮겨오기
    } else if (isLastOfParagraphCaret()) {
      const nextElement = $(selectionNode).closest(textItemSelector).next()[0]
      if (nextElement && nextElement.nodeName == 'FIGURE') {
        e.preventDefault()
        setCursorLocation(nextElement)
      }
    }
  }

  const enterController = e => {
    const selectionNode = editor.selection.getNode()
    if (selectionNode.tagName == "FIGURE") {
      e.preventDefault()
      if (!selectionNode.nextSibling || selectionNode.nextSibling.tagName != 'P') {
        editor.$(selectionNode).after('<p><br></p>')
      }
      setCursorLocation(selectionNode.nextSibling)
    }
  }

  const scrollToSelection = (elem, key) => {
    const win = editor.contentWindow
    if (key == 'ArrowDown' && elem.offsetTop + elem.offsetHeight > win.innerHeight + win.scrollY) {
      win.scrollTo(0, elem.offsetTop + elem.offsetHeight - (win.innerHeight))
    } else if (key == 'ArrowUp' && elem.offsetTop < win.scrollY) {
      win.scrollTo(0, elem.offsetTop)
    }
  }

  const removeFigureAfterSelection = selectionNode => {
    const selectionNodeNextSibling = selectionNode.nextSibling
    editor.$(selectionNode).remove()
    setCursorLocation(selectionNodeNextSibling)
    editor.undoManager.add()
  }

  const isFirstOfParagraphCaret = () => {
    const sel = editor.selection.getSel()
    const selectionNode = editor.selection.getNode()
    let textSelectNode = sel.focusNode
    if (sel.focusOffset != sel.anchorOffset) {
      return false
    }
    if (textSelectNode.nodeName == '#text') {
      if (sel.focusOffset == 0 && sel.anchorOffset == 0) {
        const textClosestItem = editor.$(textSelectNode).closest(textItemSelector)[0]
        if (isFirstChild(textSelectNode, textClosestItem)) {
          if (textSelectNode.textContent == '' && textSelectNode.nextSibling && textSelectNode.nextSibling.nodeName == 'BR') {
            textSelectNode.parentNode.removeChild(textSelectNode.nextSibling)
          }
          return true
        }
      }
    } else if (textSelectNode == selectionNode) {
      const textClosestItem = editor.$(textSelectNode).closest(textItemSelector)[0]
      if (sel.focusNode == textClosestItem && sel.focusNode.nodeName == '#text' && sel.baseOffset == 0
        && sel.focusNode == sel.focusNode.firstChild) {
        return true
      } else if (sel.focusNode == sel.focusNode && sel.focusNode.firstChild && sel.focusNode.firstChild.nodeName == '#text' 
        && sel.focusNode.firstChild.textContent == '' && sel.focusNode.firstChild.nextSibling.nodeName == 'BR') {
        sel.focusNode.removeChild(sel.focusNode.firstChild.nextSibling)
        return true
      } else if (textClosestItem == sel.focusNode && sel.focusOffset == 0) {
        return true
      }
    }
  }

  const isFirstChild = (targetNode, parentNode) => {
    if (targetNode.nodeName == 'BODY') {
      return false
    }
    if (targetNode == parentNode.firstChild) {
      return true
    } else if (targetNode.previousSibling && targetNode.previousSibling.nodeName == 'BR') {
      return false
    } else if (targetNode.previousSibling && targetNode.previousSibling.nodeName == '#text' && targetNode.previousSibling.textContent == '') {
      return isFirstChild(targetNode.previousSibling, parentNode)
    } else {
      return isFirstChild(targetNode.parentNode, parentNode)
    }
  }

  const isLastOfParagraphCaret = () => {
    const sel = editor.selection.getSel()
    const selectionNode = editor.selection.getNode()
    if (sel.focusOffset != sel.anchorOffset) {
      return false
    }
    if (sel.focusNode.nodeName == '#text') {
      if (sel.focusNode.length == sel.focusOffset) {
        const textClosestItem = editor.$(selectionNode).closest(textItemSelector)[0]
        if (isLastChild(sel.focusNode, textClosestItem)) {
          return true
        }
      }
    } else {
      const textClosestItem = editor.$(selectionNode).closest(textItemSelector)[0]
      if (sel.focusNode == textClosestItem || editor.$(sel.focusNode).closest(textItemSelector)[0] == textClosestItem) {
        const textInfo = getSelectionTextInfo(sel.focusNode)
        if (textInfo.atEnd) {
          return true
        }
      }
    }
  }

  const isLastChild = (targetNode, parentNode) => {
    if (targetNode.nodeName == 'BODY') {
      return false
    }
    if (targetNode == parentNode.lastChild) {
      return true
    } else if (targetNode.nextSibling && targetNode.nextSibling.nodeName == '#text' && targetNode.nextSibling.textContent != '') {
      return false
    } else if (targetNode.nextSibling && targetNode.nextSibling.nodeName == 'BR' && targetNode.nextSibling.nextSibling == null) {
      return true
    } else if (targetNode.nextSibling && targetNode.nextSibling.nodeName == 'BR' && targetNode.nextSibling.nextSibling.nodeName == 'BR') {
      return false
    } else if (targetNode.nextSibling && targetNode.nextSibling.nodeName == '#text' && targetNode.nextSibling.textContent == '') {
      return isLastChild(targetNode.nextSibling, parentNode)
    } else if (targetNode.nextSibling && targetNode.nextSibling.nodeName == 'BR' && targetNode.nextSibling.nextSibling.nodeName == '#text' && targetNode.nextSibling.nextSibling.textContent == '') {
      return isLastChild(targetNode.nextSibling.nextSibling, parentNode)
    } else {
      return isLastChild(targetNode.parentNode, parentNode)
    }
  }

  const getSelectionTextInfo = node => {
    const sel = editor.selection.getSel()
    let range, cloneRange, atStart = false, atEnd = false
    if (sel.rangeCount) {
      range = sel.getRangeAt(0);
      cloneRange = range.cloneRange();
      cloneRange.selectNodeContents(node);
      cloneRange.setEnd(range.startContainer, range.startOffset);
      atStart = '' === cloneRange.toString();
      cloneRange.selectNodeContents(node);
      cloneRange.setStart(range.endContainer, range.endOffset);
      atEnd = '' === cloneRange.toString();
    }
    return {
			atStart: atStart,
			atEnd: atEnd
		}
  }

  const setCursorLocation = (targetElem) => {
    if (!targetElem) return false
    editor.selection.setCursorLocation(targetElem)
    setCursorLocationFirst(targetElem)
    if (targetElem.nodeName == 'FIGURE' && targetElem.getAttribute('contenteditable') == 'false') {
      editor.selection.select(targetElem)
    }
  }

  const setCursorLocationFirst = targetElement => {
    const range = editor.selection.getRng()
    range.setStart(targetElement, 0)
    range.setEnd(targetElement, 0)
    editor.selection.setCursorLocation(targetElement)
    editor.selection.getSel().addRange(range);
  }
}

export default {
  register
}