const Tools = tinymce.util.Tools.resolve('tinymce.util.Tools');
const Factory = tinymce.util.Tools.resolve('tinymce.ui.Factory');

const getIndexedToolbars = (settings, defaultToolbar) => {
  const toolbars = [];
  for (let i = 1; i < 10; i++) {
    const toolbar_1 = settings['toolbar' + i];
    if (!toolbar_1) {
      break;
    }
    toolbars.push(toolbar_1);
  }
  const mainToolbar = settings.toolbar ? [settings.toolbar] : [defaultToolbar];
  return toolbars.length > 0 ? toolbars : mainToolbar;
};
const getToolbars = editor => {
  const toolbar = editor.getParam('toolbar');
  const defaultToolbar = 'undo redo | styleselect | bold italic | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent | link image';
  if (toolbar === false) {
    return [];
  } else if (Tools.isArray(toolbar)) {
    return Tools.grep(toolbar, toolbar => toolbar.length > 0);
  } else {
    return getIndexedToolbars(editor.settings, defaultToolbar);
  }
};

const createToolbar = (editor, items, size) => {
  const toolbarItems = [];
  let buttonGroup;
  if (!items) {
    return;
  }
  Tools.each(items.split(/[ ,]/), item => {
    let itemName;
    const bindSelectorChanged = () => {
      const selection = editor.selection;
      if (item.settings.stateSelector) {
        selection.selectorChanged(item.settings.stateSelector, state => {
          item.active(state);
        }, true);
      }
      if (item.settings.disabledStateSelector) {
        selection.selectorChanged(item.settings.disabledStateSelector, state => {
          item.disabled(state);
        });
      }
    };
    if (item === '|') {
      buttonGroup = null;
    } else {
      if (!buttonGroup) {
        buttonGroup = {
          type: 'buttongroup',
          items: []
        };
        toolbarItems.push(buttonGroup);
      }
      if (editor.buttons[item]) {
        itemName = item;
        item = editor.buttons[itemName];
        if (typeof item === 'function') {
          item = item();
        }
        item.type = item.type || 'button';
        item.size = size;
        item = Factory.create(item);
        buttonGroup.items.push(item);
        if (editor.initialized) {
          bindSelectorChanged();
        } else {
          editor.on('init', bindSelectorChanged);
        }
      }
    }
  });
  return {
    type: 'toolbar',
    layout: 'flow',
    items: toolbarItems
  };
};
const createToolbars = (editor, size) => {
  const toolbars = [];
  const addToolbar = items => {
    if (items) {
      toolbars.push(createToolbar(editor, items, size));
    }
  };
  Tools.each(getToolbars(editor), toolbar => {
    addToolbar(toolbar);
  });
  if (toolbars.length) {
    return {
      type: 'panel',
      layout: 'stack',
      classes: 'toolbar-grp',
      ariaRoot: true,
      ariaRemember: true,
      items: toolbars
    };
  }
};

export default {
  createToolbar,
  createToolbars
}