import { Replacing, ComponentApi, Gui } from '@ephox/alloy';
import { Fun, Singleton } from '@ephox/katamari';

import AndroidWebapp from '../api/AndroidWebapp';
import Styles from '../style/Styles';
import ScrollingToolbar from '../toolbar/ScrollingToolbar';
import CommonRealm from './CommonRealm';
import * as Dropup from './Dropup';
import OuterContainer from './OuterContainer';
import { SugarElement } from '../alien/TypeDefinitions';
import { MobileRealm } from '../ui/IosRealm';
import { MobileWebApp } from '../api/IosWebapp';

export default function (scrollIntoView: () => void) {
  const alloy = OuterContainer({
    classes: [ Styles.resolve('android-container') ]
  }) as Gui.GuiSystem;

  const toolbarCatalogue = {
    first: ScrollingToolbar(),
    second: ScrollingToolbar(),
    third: ScrollingToolbar(),
    fourth: ScrollingToolbar(),
  };

  const webapp = Singleton.api<MobileWebApp>();

  const switchToEdit = CommonRealm.makeEditSwitch(webapp);

  const socket = CommonRealm.makeSocket() as ComponentApi.AlloyComponent;

  const dropup = Dropup.build(Fun.noop, scrollIntoView);

  for(const toolbar in toolbarCatalogue) {
    alloy.add(toolbarCatalogue[toolbar].wrapper());
  }

  alloy.add(socket);
  alloy.add(dropup.component());

  toolbarCatalogue.third.wrapper().element().dom().style.display = "none";
  toolbarCatalogue.fourth.wrapper().element().dom().style.display = "none";

  const setToolbarGroups = function (rawGroups) {
    const mainGroups = toolbarCatalogue.second.createGroups(rawGroups.main);
    const firstGroups = toolbarCatalogue.first.createGroups(rawGroups.first);
    const thirdGroups = toolbarCatalogue.third.createGroups(rawGroups.third);
    toolbarCatalogue.second.setGroups(mainGroups);
    toolbarCatalogue.first.setGroups(firstGroups);
    toolbarCatalogue.third.setGroups(thirdGroups);
  };

  const setContextToolbar = function (rawGroups, target) {
    if(target === 3) {
      toolbarCatalogue.third.wrapper().element().dom().style.display = "flex";
      const groups = toolbarCatalogue.third.createGroups(rawGroups);
      toolbarCatalogue.third.setContextToolbar(groups);
    }
    else if(target === 4) {
      toolbarCatalogue.fourth.wrapper().element().dom().style.display = "flex";
      const groups = toolbarCatalogue.fourth.createGroups(rawGroups);
      toolbarCatalogue.fourth.setContextToolbar(groups);
    }
  };

  // You do not always want to do this.
  const focusToolbar = function () {
    toolbarCatalogue.second.focus();
  };

  const restoreToolbar = function () {
    toolbarCatalogue.third.wrapper().element().dom().style.display = "none";
    toolbarCatalogue.fourth.wrapper().element().dom().style.display = "none";
    toolbarCatalogue.third.restoreToolbar();
    toolbarCatalogue.fourth.restoreToolbar();
  };

  const closeFourthToolbar = function () {
    toolbarCatalogue.fourth.restoreToolbar();
    toolbarCatalogue.fourth.wrapper().element().dom().style.display = "none";
  }

  const init = function (spec) {
    webapp.set(
      AndroidWebapp.produce(spec)
    );
  };

  const exit = function () {
    webapp.run(function (w) {
      w.exit();
      Replacing.remove(socket, switchToEdit);
    });
  };

  const updateMode = function (readOnly) {
    CommonRealm.updateMode(socket, switchToEdit, readOnly, alloy.root());
  };

  return {
    system: Fun.constant(alloy),
    element: alloy.element as () => SugarElement,
    init,
    exit,
    setToolbarGroups,
    setContextToolbar,
    focusToolbar,
    restoreToolbar,
    closeFourthToolbar,
    updateMode,
    socket: Fun.constant(socket),
    dropup: Fun.constant(dropup)
  } as MobileRealm;
}