import { Behaviour, Button, Toggling, Unselecting, SketchSpec } from '@ephox/alloy';
import { Merger } from '@ephox/katamari';

import Receivers from '../channels/Receivers';
import Styles from '../style/Styles';
import * as UiDomFactory from '../util/UiDomFactory';

const forToolbarCommand = function (editor, command, value?) {
  return forToolbar(command, function () {
    editor.execCommand(command, false, value);
  }, { });
};

const getToggleBehaviours = function (command, selected?, value?) {
  return Behaviour.derive([
    Toggling.config({
      toggleClass: Styles.resolve('icon-selected'),
      toggleOnExecute: false,
      selected,
      aria: {
        mode: 'pressed'
      }
    }),
    Receivers.format(value || command, function (button, status) {
      const toggle = status ? Toggling.on : Toggling.off;
      toggle(button);
    })
  ]);
};

const forToolbarStateCommand = function (editor, command, value?) {
  const selected = editor.formatter.match(value || command);
  const extraBehaviours = getToggleBehaviours(command, selected, value);

  return forToolbar(command, function () {
    editor.execCommand(command, false, value);
  }, extraBehaviours);
};

// The action is not just executing the same command
const forToolbarStateAction = function (editor, clazz, command, action) {
  const extraBehaviours = getToggleBehaviours(command, false);
  return forToolbar(clazz, action, extraBehaviours);
};

const forToolbar = function (clazz, action, extraBehaviours): SketchSpec {
  const dom = '<span class="${prefix}-toolbar-button ${prefix}-icon-' + clazz + ' ${prefix}-icon" data-ke-btn-name="' + clazz + '"></span>'
  return Button.sketch({
    dom: UiDomFactory.dom(dom),
    action,

    buttonBehaviours: Merger.deepMerge(
      Behaviour.derive([
        Unselecting.config({ })
      ]),
      extraBehaviours
    )
  });
};

export default {
  forToolbar,
  forToolbarCommand,
  forToolbarStateAction,
  forToolbarStateCommand
};