import * as ToolbarWidgets from 'tinymce/themes/mobileClassic/main/ts/ui/ToolbarWidgets';
import * as UiDomFactory from 'tinymce/themes/mobileClassic/main/ts/util/UiDomFactory';
import { Toolbar, Button, Container, Behaviour, Receiving, Toggling } from '@ephox/alloy';
import Receivers from '../channels/Receivers';
import TinyChannels from '../channels/TinyChannels';
import Styles from '../style/Styles';
import { Objects } from '@ephox/boulder';

const convertToHexCode = colorString => {

  const m = /rgb\((\d*), (\d*), (\d*)\)/gi.exec(colorString)

  return '#' + m.slice(1).reduce((res, cur) => {
    let parsedHex = parseInt(cur).toString(16);
    if(parsedHex.length % 2) {
      parsedHex = '0' + parsedHex;
    }
    return res + parsedHex;
  }, "").toUpperCase();
}

const match = (color, node) => {
  if(node.nodeName !== 'SPAN') return false;
  if(color === convertToHexCode(node.style.color)) {
    return true;
  }

  return false;
}

const makeComponents = (realm, editor) => {
  const preset = editor.settings.colorPalette.color_map_preset;
  const returnee = [];
  preset.forEach(color => {
    const selected = match(color, editor.dom.getParent(editor.selection.getNode()));
    const btn = Button.sketch({
      dom: UiDomFactory.dom(`<span class="colorPalette-preset-mobile" style="background-color: ${color}"><span class="colorPalette-preset-mobile-selected"></span></span>`),
      action: () => {
        realm.system().broadcastOn([ TinyChannels.colorTouched() ], {
          color: color,
        });
        editor.formatter.apply('forecolor', { value : color });
      },
      buttonBehaviours: Behaviour.derive([
        Toggling.config({
          toggleClass: Styles.resolve('colorPalette-preset-selected'),
          toggleOnExecute: true,
          selected,
        }),
        Receivers.colorTouched(button => {
          Toggling.off(button);
        })
      ])
    });
    returnee.push(btn);
  });
  return returnee;
}

const makeContainer = (realm, editor) => {
  return Container.sketch({
    dom: UiDomFactory.dom('<div class="colorPalette-container"></div>'),
    components: makeComponents(realm, editor),
    
  })
}

const makeItems = (realm, editor) => {
  return [
    makeContainer(realm, editor)
  ]
}

const sketch = (realm, editor) => ToolbarWidgets.button(realm, 'color', () => makeItems(realm, editor), 4, () => {});

export {
  sketch
};