import * as ToolbarWidgets from 'tinymce/themes/mobileClassic/main/ts/ui/ToolbarWidgets';
import * as UiDomFactory from 'tinymce/themes/mobileClassic/main/ts/util/UiDomFactory';
import { Toolbar, Button, Container, Input, Representing, Memento, Behaviour, Toggling, Receiving } from '@ephox/alloy';
import * as LinkUtils from '../util/kLink/LinkUtils';
import Receivers from '../channels/Receivers';
import TinyChannels from '../channels/TinyChannels';
import Styles from '../style/Styles';

const getCurrentData = editor => {
  const node = editor.selection.getNode();
  if(node.nodeName === 'A') {
    return node.getAttribute('href');
  }
}

const makeComponents = editor => {
  
  const inputSpec = Memento.record(Input.sketch({
    placeholder: "URL을 입력해주세요",
    data: getCurrentData(editor)
  }));
  
  return [
    Button.sketch({
      dom: UiDomFactory.dom(`<span class="tinymce-mobile-icon tinymce-mobile-icon-unlink"></span>`),
      action: () => {
        editor.execCommand('Unlink', false);
      }
    }),    
    inputSpec.asSpec(),
    Button.sketch({
      dom: UiDomFactory.dom(`<span class="tinymce-mobile-icon tinymce-mobile-icon-link"></span>`),
      action: (button) => {
        const input = inputSpec.get(button);
        const href = Representing.getValue(input);
        if(LinkUtils.isValidHref(href)) {
          editor.execCommand('mceInsertLink', false, {
            href,
            target: '_blank',
          });
        }
      }
    }),
  ]
}

const makeContainer = editor => {
  return Container.sketch({
    dom: UiDomFactory.dom('<div class="klink-container-mobile"></div>'),
    components: makeComponents(editor),
   })
}

const makeItems = editor => {
  return [
    makeContainer(editor)
  ]
}

const sketch = (realm, editor) => ToolbarWidgets.button(realm, 'link', () => makeItems(editor), 3, realm.closeFourthToolbar);

export {
  sketch
};