import {isEmail, isMailto, isUrl} from "./formatter"


export const getAnchorElement = (editor) => {
	const selectedElm = editor.selection.getNode() || editor.selection.getStart();
	return editor.dom.getParent(selectedElm, 'a[href]')
}


export const getAnchorText = (selection, anchorElm) => {
	var text = anchorElm ? (anchorElm.innerText || anchorElm.textContent) : selection.getContent({ format: 'text' });
	return trimCaretContainers(text);
}

export const trimCaretContainers = (text) => {
	return text.replace(/\uFEFF/g, '');
}


export const makeValidHref = (url) => {
	if (isUrl(url)) {
		return url
	}

	if (isUrl("http://" + url)) {
		return "http://" + url
	}

	if (isMailto(url)) {
		return url
	}

	if (isEmail(url)) {
		return "mailto:" + url
	}

	return ''
}

export const isValidHref = (url) => {
	const href = makeValidHref(url)
	return href !== ''
}
