import { Assertions, Pipeline } from '@ephox/agar';
import { Obj } from '@ephox/katamari';
import { LegacyUnit, TinyLoader } from '@ephox/mcagar';
import Theme from 'src/themes/classic/main/ts/Theme';
import { UnitTest } from '@ephox/bedrock';

UnitTest.asynctest('browser.FormatterApplyTest', function () {
  const success = arguments[arguments.length - 2];
  const failure = arguments[arguments.length - 1];
  const suite = LegacyUnit.createSuite();

  Theme();

  const getContent = function (editor) {
    return editor.getContent().toLowerCase().replace(/[\r]+/g, '');
  };

  suite.test('apply inline to a list', function (editor) {
    editor.focus();
    editor.formatter.register('format', {
      inline: 'b',
      toggle: false
    });
    editor.getBody().innerHTML = '<p>1234</p><ul><li>first element</li><li>second element</li></ul><p>5678</p>';
    const rng = editor.dom.createRng();
    rng.setStart(editor.dom.select('p')[0].firstChild, 0);
    rng.setEnd(editor.dom.select('p')[1].firstChild, 4);
    editor.selection.setRng(rng);
    editor.formatter.apply('format');
    LegacyUnit.equal(
      getContent(editor),
      '<p><b>1234</b></p><ul><li><b>first element</b></li><li><b>second element</b></li></ul><p><b>5678</b></p>',
      'selection of a list'
    );
  });

  TinyLoader.setup(function (editor, onSuccess, onFailure) {
    Pipeline.async({}, suite.toSteps(editor), onSuccess, onFailure);
  }, {
    indent: false,
    extended_valid_elements: 'b/strong,s/strike,s/del,i/em,iframe[mapdata|src|id|width|height|frameborder|scrolling|data-*]',
    entities: 'raw',
    theme: 'classic',
    convert_fonts_to_spans: false,
    convert_urls: false,
    relative_urls: false,
    forced_root_block: false,
    // valid_styles: {
    //   '*': 'color,font-size,font-family,background-color,font-weight,font-style,text-decoration,float,' +
    //       'margin,margin-top,margin-right,margin-bottom,margin-left,display,text-align'
    // },
    valid_children : 'p[span|b|u|i|s|a|#text|br],-h2[img|div|figure|b],figcaption[#text],figure[img|figcaption|br|a],-span[img]',
    skin_url: 'src/skins/classic'
  }, success, failure);
});
